# Copyright (c) 2015 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from functionaltests.api import base
from functionaltests.api.v1.behaviors import consumer_behaviors
from functionaltests.api.v1.behaviors import container_behaviors
from functionaltests.api.v1.behaviors import order_behaviors
from functionaltests.api.v1.behaviors import quota_behaviors
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.models import consumer_model
from functionaltests.api.v1.models import container_models
from functionaltests.api.v1.models import order_models
from functionaltests.api.v1.models import quota_models
from functionaltests.api.v1.models import secret_models
from functionaltests.common import config


CONF = config.get_config()
admin_b = CONF.rbac_users.admin_b
service_admin = CONF.identity.service_admin


@testtools.testcase.attr('no_parallel')
class QuotaEnforcementTestCase(base.TestCase):

    def setUp(self):
        super(QuotaEnforcementTestCase, self).setUp()
        self.quota_behaviors = quota_behaviors.QuotaBehaviors(self.client)
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.container_behaviors = container_behaviors.ContainerBehaviors(
            self.client)
        self.order_behaviors = order_behaviors.OrderBehaviors(self.client)
        self.consumer_behaviors = consumer_behaviors.ConsumerBehaviors(
            self.client)

        self.secret_data = self.get_default_secret_data()
        self.quota_data = self.get_default_quota_data()
        self.project_id = self.quota_behaviors.get_project_id_from_name(
            admin_b)
        self.order_secrets = []
        self.root_ca_ref = None
        self.test_order_sent = False

    def tearDown(self):
        self.quota_behaviors.delete_all_created_quotas()
        self.consumer_behaviors.delete_all_created_consumers()
        self.container_behaviors.delete_all_created_containers()
        self.secret_behaviors.delete_all_created_secrets()
        for secret_ref in self.order_secrets:
            resp = self.secret_behaviors.delete_secret(
                secret_ref, user_name=admin_b)
            self.assertEqual(204, resp.status_code)
        self.order_behaviors.delete_all_created_orders()
        super(QuotaEnforcementTestCase, self).tearDown()

    def test_secrets_unlimited(self):
        self.set_quotas('secrets', -1)
        self.create_secrets(count=5)

    def test_secrets_disabled(self):
        self.set_quotas('secrets', 0)
        self.create_secrets(expected_return=403)

    def test_secrets_limited_one(self):
        self.set_quotas('secrets', 1)
        self.create_secrets(count=1)
        self.create_secrets(expected_return=403)

    def test_secrets_limited_five(self):
        self.set_quotas('secrets', 5)
        self.create_secrets(count=5)
        self.create_secrets(expected_return=403)

    def test_containers_unlimited(self):
        self.set_quotas('containers', -1)
        self.create_containers(count=5)

    def test_containers_disabled(self):
        self.set_quotas('containers', 0)
        self.create_containers(expected_return=403)

    def test_containers_limited_one(self):
        self.set_quotas('containers', 1)
        self.create_containers(count=1)
        self.create_containers(expected_return=403)

    def test_containers_limited_five(self):
        self.set_quotas('containers', 5)
        self.create_containers(count=5)
        self.create_containers(expected_return=403)

    def test_orders_unlimited(self):
        self.set_quotas('orders', -1)
        self.create_orders(count=5)

    def test_orders_disabled(self):
        self.set_quotas('orders', 0)
        self.create_orders(expected_return=403)

    def test_orders_limited_one(self):
        self.set_quotas('orders', 1)
        self.create_orders(count=1)
        self.create_orders(expected_return=403)

    def test_orders_limited_five(self):
        self.set_quotas('orders', 5)
        self.create_orders(count=5)
        self.create_orders(expected_return=403)

    def test_consumers_unlimited(self):
        self.set_quotas('consumers', -1)
        self.create_consumers(count=5)

    def test_consumers_disabled(self):
        self.set_quotas('consumers', 0)
        self.create_consumers(expected_return=403)

    def test_consumers_limited_one(self):
        self.set_quotas('consumers', 1)
        self.create_consumers(count=1)
        self.create_consumers(expected_return=403)

    def test_consumers_limited_five(self):
        self.set_quotas('consumers', 5)
        self.create_consumers(count=5)
        self.create_consumers(expected_return=403)

# ----------------------- Helper Functions ---------------------------

    def get_default_quota_data(self):
        return {"project_quotas":
                {"secrets": -1,
                 "orders": -1,
                 "containers": -1,
                 "consumers": -1}}

    def set_quotas(self, resource, quota):
        """Utility function to set resource quotas"""
        self.quota_data["project_quotas"][resource] = quota
        request_model = quota_models.ProjectQuotaRequestModel(
            **self.quota_data)
        resp = self.quota_behaviors.set_project_quotas(self.project_id,
                                                       request_model,
                                                       user_name=service_admin)
        self.assertEqual(204, resp.status_code)

    def get_default_secret_data(self):
        return {
            "name": "AES key",
            "expiration": "2050-02-28T19:14:44.180394",
            "algorithm": "aes",
            "bit_length": 256,
            "mode": "cbc",
            "payload": "Z0Y2K2xMb0Yzb2hBOWFQUnB0KzZiUT09",
            "payload_content_type": "application/octet-stream",
            "payload_content_encoding": "base64",
        }

    def create_secrets(self, count=1, expected_return=201):
        """Utility function to create secrets"""
        secret_ref = None
        for _ in range(count):
            test_model = secret_models.SecretModel(**self.secret_data)
            resp, secret_ref = self.secret_behaviors.create_secret(
                test_model, user_name=admin_b)
            self.assertEqual(expected_return, resp.status_code)
        return secret_ref

    def get_container_req(self, secret_ref):
        return {"name": "test_container",
                "type": "generic",
                "secret_refs": [{'name': 'secret1', 'secret_ref': secret_ref}]}

    def create_containers(self, count=1, expected_return=201):
        """Utility function to create containers"""
        container_ref = None
        for _ in range(count):
            secret_ref = self.create_secrets()
            test_model = container_models.ContainerModel(
                **self.get_container_req(secret_ref))
            resp, container_ref = self.container_behaviors.create_container(
                test_model, user_name=admin_b)
            self.assertEqual(expected_return, resp.status_code)
        return container_ref

    def get_default_order_data(self):
        return {'type': 'key',
                "meta": {
                    "name": "barbican functional test order name",
                    "algorithm": "aes",
                    "bit_length": 256,
                    "mode": "cbc"}}

    def create_orders(self, count=1, expected_return=202):
        """Utility function to create orders"""
        for _ in range(count):
            order_data = self.get_default_order_data()
            test_model = order_models.OrderModel(**order_data)
            resp, order_ref = self.order_behaviors.create_order(
                test_model, user_name=admin_b)
            self.assertEqual(expected_return, resp.status_code)
            if resp.status_code == 202:
                order_resp = self.order_behaviors.get_order(
                    order_ref, user_name=admin_b)
                self.assertEqual(200, order_resp.status_code)
                self.order_secrets.append(order_resp.model.secret_ref)

    def get_default_consumer_data(self):
        return {"name": "consumer_name",
                "URL": "consumer_url"}

    def create_consumers(self, count=1, expected_return=200):
        """Utility function to create consumers"""
        for _ in range(count):
            container_ref = self.create_containers()
            model = consumer_model.ConsumerModel(
                **self.get_default_consumer_data())
            resp, consumer_dat = self.consumer_behaviors.create_consumer(
                model, container_ref, user_name=admin_b)
            self.assertEqual(expected_return, resp.status_code)



# Copyright (c) 2015 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import uuidutils

from barbican.tests import utils
from functionaltests.api import base
from functionaltests.api.v1.behaviors import quota_behaviors
from functionaltests.api.v1.models import quota_models
from functionaltests.common import config


CONF = config.get_config()
admin_a = CONF.rbac_users.admin_a
creator_a = CONF.rbac_users.creator_a
observer_a = CONF.rbac_users.observer_a
auditor_a = CONF.rbac_users.auditor_a
service_admin = CONF.identity.service_admin

test_data_rbac_get_quotas = {
    'with_service_admin': {'user': service_admin, 'admin': service_admin,
                           'expected_return': 200},
    'with_admin_a': {'user': admin_a, 'admin': admin_a,
                     'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'admin': admin_a,
                       'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'admin': admin_a,
                        'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'admin': admin_a,
                       'expected_return': 200},
}


test_data_rbac_get_project_quotas = {
    'with_service_admin': {'user': service_admin, 'admin': service_admin,
                           'expected_return': 200},
    'with_admin_a': {'user': admin_a, 'admin': admin_a,
                     'expected_return': 403},
    'with_creator_a': {'user': creator_a, 'admin': admin_a,
                       'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'admin': admin_a,
                        'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'admin': admin_a,
                       'expected_return': 403},
}

test_data_rbac_set_project_quotas = {
    'with_service_admin': {'user': service_admin, 'admin': service_admin,
                           'expected_return': 204},
    'with_admin_a': {'user': admin_a, 'admin': admin_a,
                     'expected_return': 403},
    'with_creator_a': {'user': creator_a, 'admin': admin_a,
                       'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'admin': admin_a,
                        'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'admin': admin_a,
                       'expected_return': 403},
}


test_data_rbac_delete_project_quotas = {
    'with_service_admin': {'user': service_admin, 'admin': service_admin,
                           'expected_return': 204},
    'with_admin_a': {'user': admin_a, 'admin': admin_a,
                     'expected_return': 403},
    'with_creator_a': {'user': creator_a, 'admin': admin_a,
                       'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'admin': admin_a,
                        'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'admin': admin_a,
                       'expected_return': 403},
}


def get_set_project_quotas_request():
    return {"project_quotas":
            {"secrets": 50,
             "orders": 10,
             "containers": 20}}


@utils.parameterized_test_case
class RBACQuotasTestCase(base.TestCase):
    """Functional tests exercising RBAC Policies"""
    def setUp(self):
        super(RBACQuotasTestCase, self).setUp()
        self.behaviors = quota_behaviors.QuotaBehaviors(self.client)

    def tearDown(self):
        self.behaviors.delete_all_created_quotas()
        super(RBACQuotasTestCase, self).tearDown()

    @utils.parameterized_dataset(test_data_rbac_get_quotas)
    def test_rbac_get_quotas(self, user, admin, expected_return):
        """Test RBAC for get quotas

        Issue a get quotas and verify that the correct
        http return code comes back for the specified user.

        :param user: the user who will attempt to do the get
        :param admin: the admin of the group owning quotas
        :param expected_return: the expected http return code
        """
        resp = self.behaviors.get_quotas(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @utils.parameterized_dataset(test_data_rbac_get_project_quotas)
    def test_rbac_get_project_quotas(self, user, admin, expected_return):
        """Test RBAC for get project quotas

        Issue a get quotas and verify that the correct
        http return code comes back for the specified user.

        :param user: the user who will attempt to do the get
        :param admin: the admin of the group owning quotas
        :param expected_return: the expected http return code
        """
        resp, _, _, _ = self.behaviors.get_project_quotas_list(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @utils.parameterized_dataset(test_data_rbac_set_project_quotas)
    def test_rbac_set_project_quotas(self, user, admin, expected_return):
        """Test RBAC for set project quotas

        Issue a set project quotas and verify that the correct
        http return code comes back for the specified user.

        :param user: the user who will attempt to do the set
        :param admin: the admin of the group owning quotas
        :param expected_return: the expected http return code
        """
        request_model = quota_models.ProjectQuotaRequestModel(
            **get_set_project_quotas_request())
        project_id = uuidutils.generate_uuid()
        resp = self.behaviors.set_project_quotas(project_id,
                                                 request_model,
                                                 user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @utils.parameterized_dataset(test_data_rbac_delete_project_quotas)
    def test_rbac_delete_project_quotas(self, user, admin, expected_return):
        """Test RBAC for delete project quotas

        Issue a set project quotas and verify that the correct
        http return code comes back for the specified user.

        :param user: the user who will attempt to do the delete
        :param admin: the admin of the group owning quotas
        :param expected_return: the expected http return code
        """
        request_model = quota_models.ProjectQuotaRequestModel(
            **get_set_project_quotas_request())
        project_id = uuidutils.generate_uuid()
        resp = self.behaviors.set_project_quotas(project_id,
                                                 request_model,
                                                 user_name=service_admin)
        resp = self.behaviors.delete_project_quotas(project_id,
                                                    user_name=user)
        self.assertEqual(expected_return, resp.status_code)



# Copyright (c) 2014 Red Hat, Inc.
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import pecan
from urllib import parse

from barbican import api
from barbican.api import controllers
from barbican.common import exception
from barbican.common import hrefs
from barbican.common import utils
from barbican.common import validators
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repo

LOG = utils.getLogger(__name__)


def _transport_key_not_found():
    """Throw exception indicating transport key not found."""
    pecan.abort(404, u._('Not Found. Transport Key not found.'))


def _invalid_transport_key_id():
    """Throw exception indicating transport key id is invalid."""
    pecan.abort(404, u._('Not Found. Provided transport key id is invalid.'))


class TransportKeyController(controllers.ACLMixin):
    """Handles transport key retrieval requests."""

    def __init__(self, transport_key_id, transport_key_repo=None):
        LOG.debug('=== Creating TransportKeyController ===')
        super().__init__()
        self.transport_key_id = transport_key_id
        self.repo = transport_key_repo or repo.TransportKeyRepo()

    @pecan.expose(generic=True)
    def index(self, external_project_id, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET')
    @controllers.handle_exceptions(u._('Transport Key retrieval'))
    @controllers.enforce_rbac('transport_key:get')
    def on_get(self, external_project_id):
        LOG.debug("== Getting transport key for %s", external_project_id)
        transport_key = self.repo.get(entity_id=self.transport_key_id)
        if not transport_key:
            _transport_key_not_found()

        pecan.override_template('json', 'application/json')
        return transport_key

    @index.when(method='DELETE')
    @controllers.handle_exceptions(u._('Transport Key deletion'))
    @controllers.enforce_rbac('transport_key:delete')
    def on_delete(self, external_project_id, **kwargs):
        LOG.debug("== Deleting transport key ===")
        try:
            self.repo.delete_entity_by_id(
                entity_id=self.transport_key_id,
                external_project_id=external_project_id)
            # TODO(alee) response should be 204 on success
            # pecan.response.status = 204
        except exception.NotFound:
            LOG.exception('Problem deleting transport_key')
            _transport_key_not_found()


class TransportKeysController(controllers.ACLMixin):
    """Handles transport key list requests."""

    def __init__(self, transport_key_repo=None):
        LOG.debug('Creating TransportKeyController')
        super().__init__()
        self.repo = transport_key_repo or repo.TransportKeyRepo()
        self.validator = validators.NewTransportKeyValidator()

    @pecan.expose()
    def _lookup(self, transport_key_id, *remainder):
        if not utils.validate_id_is_uuid(transport_key_id):
            _invalid_transport_key_id()
        return TransportKeyController(transport_key_id, self.repo), remainder

    @pecan.expose(generic=True)
    def index(self, external_project_id, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @controllers.handle_exceptions(u._('Transport Key(s) retrieval'))
    @controllers.enforce_rbac('transport_keys:get')
    def on_get(self, external_project_id, **kw):
        LOG.debug('Start transport_keys on_get')

        plugin_name = kw.get('plugin_name', None)
        if plugin_name is not None:
            plugin_name = parse.unquote_plus(plugin_name)

        result = self.repo.get_by_create_date(
            plugin_name=plugin_name,
            offset_arg=kw.get('offset', 0),
            limit_arg=kw.get('limit', None),
            suppress_exception=True
        )

        transport_keys, offset, limit, total = result

        if not transport_keys:
            transport_keys_resp_overall = {'transport_keys': [],
                                           'total': total}
        else:
            transport_keys_resp = [
                hrefs.convert_transport_key_to_href(s.id)
                for s in transport_keys
            ]
            transport_keys_resp_overall = hrefs.add_nav_hrefs(
                'transport_keys',
                offset,
                limit,
                total,
                {'transport_keys': transport_keys_resp}
            )
            transport_keys_resp_overall.update({'total': total})

        return transport_keys_resp_overall

    @index.when(method='POST', template='json')
    @controllers.handle_exceptions(u._('Transport Key Creation'))
    @controllers.enforce_rbac('transport_keys:post')
    @controllers.enforce_content_types(['application/json'])
    def on_post(self, external_project_id, **kwargs):
        LOG.debug('Start transport_keys on_post')

        # TODO(alee) POST should determine the plugin name and call the
        # relevant get_transport_key() call.  We will implement this once
        # we figure out how the plugins will be enumerated.

        data = api.load_body(pecan.request, validator=self.validator)

        new_key = models.TransportKey(data.get('plugin_name'),
                                      data.get('transport_key'))

        self.repo.create_from(new_key)

        url = hrefs.convert_transport_key_to_href(new_key.id)
        LOG.debug('URI to transport key is %s', url)

        pecan.response.status = 201
        pecan.response.headers['Location'] = url

        return {'transport_key_ref': url}



# (c) Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
from unittest import mock

from oslo_utils import uuidutils

from barbican.common import config
from barbican.common import exception
from barbican.model import models
from barbican.model import repositories
from barbican.plugin.crypto import base
from barbican.plugin.crypto import manager as cm
from barbican.plugin.crypto import p11_crypto
from barbican.plugin.crypto import simple_crypto
from barbican.plugin.interface import secret_store
from barbican.plugin import kmip_secret_store as kss
from barbican.plugin import store_crypto
from barbican.plugin.util import multiple_backends
from barbican.tests import utils as test_utils


class MockedManager(object):

    NAME_PREFIX = "friendly_"

    def __init__(self, names, enabled=True,
                 plugin_lookup_field='store_plugin'):
        ExtTuple = collections.namedtuple('ExtTuple', ['name', 'obj'])
        self.extensions = []
        for name in names:
            m = mock.MagicMock()
            m.get_plugin_name.return_value = self.NAME_PREFIX + name
            new_extension = ExtTuple(name, m)
            self.extensions.append(new_extension)
        self.global_default_store_dict = None
        self.parsed_stores = multiple_backends.read_multiple_backends_config()


class WhenReadingMultipleBackendsConfig(test_utils.MultipleBackendsTestCase):

    def test_successful_conf_read(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True,
                                global_default_index=1)

        stores = multiple_backends.read_multiple_backends_config()

        self.assertEqual(len(ss_plugins), len(stores))
        self.assertEqual('ss_p1', stores[0].store_plugin)
        self.assertEqual('cr_p1', stores[0].crypto_plugin)
        self.assertEqual(False, stores[0].global_default)
        self.assertEqual('ss_p2', stores[1].store_plugin)
        self.assertEqual('cr_p2', stores[1].crypto_plugin)
        self.assertTrue(stores[1].global_default)
        self.assertEqual('ss_p3', stores[2].store_plugin)
        self.assertEqual('cr_p3', stores[2].crypto_plugin)
        self.assertEqual(False, stores[2].global_default)

    def test_fail_when_store_plugin_name_missing(self):
        ss_plugins = ['ss_p1', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)

        self.assertRaises(exception.MultipleStorePluginValueMissing,
                          multiple_backends.read_multiple_backends_config)

    def test_fail_when_store_plugin_name_is_blank(self):
        ss_plugins = ['ss_p1', '', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)

        self.assertRaises(exception.MultipleStorePluginValueMissing,
                          multiple_backends.read_multiple_backends_config)

    def test_successful_conf_read_when_crypto_plugin_name_is_missing(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)

        stores = multiple_backends.read_multiple_backends_config()
        self.assertEqual(len(ss_plugins), len(stores))

    def test_conf_read_when_multiple_plugin_disabled(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=False)

        stores = multiple_backends.read_multiple_backends_config()
        self.assertIsNone(stores)

    def test_successful_conf_read_when_crypto_plugin_name_is_blank(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', '', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)

        stores = multiple_backends.read_multiple_backends_config()
        self.assertEqual(len(ss_plugins), len(stores))
        self.assertEqual('', stores[1].crypto_plugin)

    def test_fail_when_global_default_not_specified(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True,
                                global_default_index=-1)

        self.assertRaises(exception.MultipleStoreIncorrectGlobalDefault,
                          multiple_backends.read_multiple_backends_config)

    def test_fail_when_stores_lookup_suffix_missing_when_enabled(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True,
                                global_default_index=0)

        conf = config.get_module_config('secretstore')
        conf.set_override("stores_lookup_suffix", [], group='secretstore')
        self.assertRaises(exception.MultipleSecretStoreLookupFailed,
                          multiple_backends.read_multiple_backends_config)

    def test_fail_when_secretstore_section_missing(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True,
                                global_default_index=-1)
        ss_conf = config.get_module_config('secretstore')

        existing_value = ss_conf.secretstore.stores_lookup_suffix
        existing_value.append('unknown_section')

        ss_conf.set_override('stores_lookup_suffix', existing_value,
                             'secretstore')

        self.assertRaises(exception.MultipleStorePluginValueMissing,
                          multiple_backends.read_multiple_backends_config)


class WhenInvokingSyncSecretStores(test_utils.MultipleBackendsTestCase):

    def setUp(self):
        super(WhenInvokingSyncSecretStores, self).setUp()

    def test_successful_syncup_no_existing_secret_stores(self):

        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3', 'ss_p4', 'ss_p5']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3', 'cr_p4', 'cr_p5']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        default_secret_store = multiple_backends.\
            get_global_default_secret_store()
        self.assertEqual('ss_p1', default_secret_store.store_plugin)
        self.assertEqual('cr_p1', default_secret_store.crypto_plugin)
        self.assertEqual(MockedManager.NAME_PREFIX + 'cr_p1',
                         default_secret_store.name)

        ss_db_entries = repositories.get_secret_stores_repository().get_all()
        self.assertEqual(5, len(ss_db_entries))

    def test_syncup_with_existing_secret_stores(self):

        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3', 'ss_p4', 'ss_p5']
        cr_plugins = ['cr_p1', '', 'cr_p3', 'cr_p4', 'cr_p5']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        ss_db_entries = repositories.get_secret_stores_repository().get_all()
        self.assertEqual(5, len(ss_db_entries))

        # check friendly name for the case when crypto plugin is not there
        ss_db_entry = self._get_secret_store_entry('ss_p2', None)
        self.assertIsNotNone(ss_db_entry)
        self.assertEqual(MockedManager.NAME_PREFIX + 'ss_p2',
                         ss_db_entry.name)

        ss_plugins = ['ss_p3', 'ss_p4', 'ss_p5', 'ss_p6']
        cr_plugins = ['cr_p3', 'cr_p4', 'cr_p5', 'cr_p6']
        # update conf and re-run sync store
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)

        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        ss_db_entry = self._get_secret_store_entry('ss_p2', 'cr_p2')
        self.assertIsNone(ss_db_entry)

        ss_db_entry = self._get_secret_store_entry('ss_p6', 'cr_p6')
        self.assertIsNotNone(ss_db_entry)

        default_secret_store = multiple_backends.\
            get_global_default_secret_store()
        self.assertEqual('ss_p3', default_secret_store.store_plugin)
        self.assertEqual('cr_p3', default_secret_store.crypto_plugin)
        self.assertEqual(MockedManager.NAME_PREFIX + 'cr_p3',
                         default_secret_store.name)
        ss_db_entries = repositories.get_secret_stores_repository().get_all()
        self.assertEqual(4, len(ss_db_entries))

    def test_syncup_modify_global_default(self):

        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3', 'ss_p4', 'ss_p5']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3', 'cr_p4', 'cr_p5']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        global_secret_store = multiple_backends.\
            get_global_default_secret_store()
        self.assertEqual('ss_p1', global_secret_store.store_plugin)
        self.assertEqual('cr_p1', global_secret_store.crypto_plugin)
        self.assertEqual(MockedManager.NAME_PREFIX + 'cr_p1',
                         global_secret_store.name)

        ss_plugins = ['ss_p9', 'ss_p4', 'ss_p5']
        cr_plugins = ['cr_p9', 'cr_p4', 'cr_p5']
        # update conf and re-run sync store
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        global_secret_store = multiple_backends.\
            get_global_default_secret_store()
        self.assertEqual('ss_p9', global_secret_store.store_plugin)
        self.assertEqual('cr_p9', global_secret_store.crypto_plugin)
        self.assertEqual(MockedManager.NAME_PREFIX + 'cr_p9',
                         global_secret_store.name)

    def test_syncup_with_store_and_crypto_plugins_count_mismatch(self):

        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3', 'ss_p4']
        cr_plugins = ['cr_p1', '', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        # empty crypto_plugin name maps to None in database entry
        ss_db_entry = self._get_secret_store_entry('ss_p2', None)
        self.assertIsNotNone(ss_db_entry)
        ss_db_entry = self._get_secret_store_entry('ss_p2', '')
        self.assertIsNone(ss_db_entry)

        # missing crypto plugin name maps to None in database entry
        ss_db_entry = self._get_secret_store_entry('ss_p4', None)
        self.assertIsNotNone(ss_db_entry)

    def test_syncup_delete_secret_store_with_preferred_project_using_it(self):
        """Removing secret store will fail if its defined as preferred store.

        """
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3', 'ss_p4']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3', 'cr_p4']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        secretstore_manager = MockedManager(ss_plugins)
        crypto_manager = MockedManager(cr_plugins)
        multiple_backends.sync_secret_stores(secretstore_manager,
                                             crypto_manager)

        with mock.patch('barbican.model.repositories.'
                        'get_project_secret_store_repository') as ps_repo:
            # Mocking with 2 projects as using preferred secret store
            ps_repo.get_count_by_secret_store.return_value = 2

            ss_plugins = ['ss_p3', 'ss_p4']
            cr_plugins = ['cr_p3', 'cr_p4']
            self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
            secretstore_manager = MockedManager(ss_plugins)
            crypto_manager = MockedManager(cr_plugins)

            self.assertRaises(exception.MultipleStorePluginStillInUse,
                              multiple_backends.sync_secret_stores,
                              secretstore_manager, crypto_manager)

    def test_get_global_default_store_when_multiple_backends_disabled(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=False)

        default_store = multiple_backends.get_global_default_secret_store()
        self.assertIsNone(default_store)


class TestGetApplicablePlugins(test_utils.MultipleBackendsTestCase):

    def setUp(self):
        super(TestGetApplicablePlugins, self).setUp()

    def test_get_when_project_preferred_plugin_is_set(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        ss_manager = MockedManager(ss_plugins)
        project_id = uuidutils.generate_uuid(dashed=False)

        with mock.patch('barbican.model.repositories.ProjectSecretStoreRepo.'
                        'get_secret_store_for_project') as pref_func:

            # set preferred secret store to one of value in config
            m_dict = {'store_plugin': 'ss_p3'}
            m_rec = mock.MagicMock()
            m_rec.secret_store.to_dict_fields.return_value = m_dict
            pref_func.return_value = m_rec

            objs = multiple_backends.get_applicable_store_plugins(
                ss_manager, project_id, None)
            self.assertIn(project_id, pref_func.call_args_list[0][0])
            self.assertIsInstance(objs, list)
            self.assertEqual(1, len(objs))
            self.assertIn('ss_p3', objs[0].get_plugin_name())

    def test_get_when_project_preferred_plugin_is_not_found_in_conf(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True)
        ss_manager = MockedManager(ss_plugins)

        project_id = uuidutils.generate_uuid(dashed=False)

        with mock.patch('barbican.model.repositories.ProjectSecretStoreRepo.'
                        'get_secret_store_for_project') as pref_func:

            # set preferred secret store value which is not defined in config
            m_dict = {'store_plugin': 'old_preferred_plugin'}
            m_rec = mock.MagicMock()
            m_rec.secret_store.to_dict_fields.return_value = m_dict
            pref_func.return_value = m_rec

            self.assertRaises(exception.MultipleStorePreferredPluginMissing,
                              multiple_backends.get_applicable_store_plugins,
                              ss_manager, project_id, None)
            self.assertIn(project_id, pref_func.call_args_list[0][0])

    def test_get_when_project_preferred_plugin_not_set_then_default_used(self):
        ss_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        # setting second plugin to be global default
        self.init_via_conf_file(ss_plugins, cr_plugins, enabled=True,
                                global_default_index=1)
        cr_manager = MockedManager(cr_plugins,
                                   plugin_lookup_field='crypto_plugin')
        project_id = uuidutils.generate_uuid(dashed=False)

        with mock.patch('barbican.plugin.util.multiple_backends.'
                        'get_global_default_secret_store') as gd_func:

            m_dict = {'crypto_plugin': 'cr_p2'}
            gd_func.return_value.to_dict_fields.return_value = m_dict
            objs = multiple_backends.get_applicable_crypto_plugins(cr_manager,
                                                                   project_id,
                                                                   None)
            gd_func.assert_called_once_with()
            self.assertIsInstance(objs, list)
            self.assertEqual(1, len(objs))
            self.assertIn('cr_p2', objs[0].get_plugin_name())

            # call again with no project_id set
            objs = multiple_backends.get_applicable_crypto_plugins(cr_manager,
                                                                   None, None)
            gd_func.assert_called_once_with()
            self.assertIsInstance(objs, list)
            self.assertEqual(1, len(objs))
            self.assertIn('cr_p2', objs[0].get_plugin_name())

    def test_get_applicable_store_plugins_when_multiple_backend_not_enabled(
            self):

        ss_config = config.get_module_config('secretstore')
        ss_plugins = ['ss_p11', 'ss_p22', 'ss_p33', 'ss_p44']
        ss_conf_plugins = ['ss_p1', 'ss_p2', 'ss_p3']
        cr_conf_plugins = ['cr_p1', 'cr_p2', 'cr_p3']
        self.init_via_conf_file(ss_conf_plugins, cr_conf_plugins,
                                enabled=False)
        ss_manager = MockedManager(ss_plugins)

        ss_config.set_override("enabled_secretstore_plugins",
                               ss_plugins, group='secretstore')

        objs = multiple_backends.get_applicable_store_plugins(ss_manager, None,
                                                              None)
        self.assertEqual(4, len(objs))


@test_utils.parameterized_test_case
class TestPluginsGenerateStoreAPIMultipleBackend(
        test_utils.MultipleBackendsTestCase):

    backend_dataset = {
        "db_backend": [{
            'store_plugins': ['store_crypto', 'kmip_plugin', 'store_crypto'],
            'crypto_plugins': ['simple_crypto', '', 'p11_crypto'],
            'default_store_class': store_crypto.StoreCryptoAdapterPlugin,
            'default_crypto_class': simple_crypto.SimpleCryptoPlugin
        }],
        "kmip": [{
            'store_plugins': ['kmip_plugin', 'store_crypto', 'store_crypto'],
            'crypto_plugins': ['', 'p11_crypto', 'simple_crypto'],
            'default_store_class': kss.KMIPSecretStore,
            'default_crypto_class': None
        }],
        "pkcs11": [{
            'store_plugins': ['store_crypto', 'store_crypto', 'kmip_plugin'],
            'crypto_plugins': ['p11_crypto', 'simple_crypto', ''],
            'default_store_class': store_crypto.StoreCryptoAdapterPlugin,
            'default_crypto_class': p11_crypto.P11CryptoPlugin
        }]
    }

    def setUp(self):
        super(TestPluginsGenerateStoreAPIMultipleBackend, self).setUp()

    def _create_project(self):
        session = repositories.get_project_repository().get_session()

        project = models.Project()
        project.external_id = ("keystone_project_id" +
                               uuidutils.generate_uuid(dashed=False))
        project.save(session=session)
        return project

    def _create_project_store(self, project_id, secret_store_id):
        proj_store_repo = repositories.get_project_secret_store_repository()
        session = proj_store_repo.get_session()

        proj_model = models.ProjectSecretStore(project_id, secret_store_id)

        proj_s_store = proj_store_repo.create_from(proj_model, session)
        proj_s_store.save(session=session)
        return proj_s_store

    @test_utils.parameterized_dataset(backend_dataset)
    def test_no_preferred_default_plugin(self, dataset):
        """Check name, plugin and crypto class used for default secret store

        Secret store name is crypto class plugin name if defined otherwise user
        friendly name is derived from store class plugin name
        """

        self.init_via_conf_file(dataset['store_plugins'],
                                dataset['crypto_plugins'],
                                enabled=True)

        with mock.patch('barbican.plugin.crypto.p11_crypto.P11CryptoPlugin.'
                        '_create_pkcs11'), \
                mock.patch('kmip.pie.client.ProxyKmipClient'):
            manager = secret_store.SecretStorePluginManager()

        keySpec = secret_store.KeySpec(secret_store.KeyAlgorithm.AES, 128)

        plugin_found = manager.get_plugin_store(keySpec)
        self.assertIsInstance(plugin_found,
                              dataset['default_store_class'])

        global_secret_store = multiple_backends.\
            get_global_default_secret_store()

        if dataset['default_crypto_class']:
            crypto_plugin = cm.get_manager().get_plugin_store_generate(
                base.PluginSupportTypes.ENCRYPT_DECRYPT)
            self.assertIsInstance(crypto_plugin,
                                  dataset['default_crypto_class'])

            # make sure secret store name is same as crypto class friendly name
            # as store_plugin class is not direct impl of SecretStoreBase
            self.assertEqual(global_secret_store.name,
                             crypto_plugin.get_plugin_name())
        else:  # crypto class is not used
            # make sure secret store name is same as store plugin class
            # friendly name
            self.assertEqual(global_secret_store.name,
                             plugin_found.get_plugin_name())
            # error raised for no crypto plugin
            self.assertRaises(base.CryptoPluginNotFound,
                              cm.get_manager().get_plugin_store_generate,
                              base.PluginSupportTypes.ENCRYPT_DECRYPT)

    @test_utils.parameterized_dataset(backend_dataset)
    def test_project_preferred_default_plugin(self, dataset):
        """Check project preferred behavior with different global default"""

        self.init_via_conf_file(dataset['store_plugins'],
                                dataset['crypto_plugins'],
                                enabled=True)

        with mock.patch('barbican.plugin.crypto.p11_crypto.P11CryptoPlugin.'
                        '_create_pkcs11'), \
                mock.patch('kmip.pie.client.ProxyKmipClient'):
            manager = secret_store.SecretStorePluginManager()

        pkcs11_secret_store = self._get_secret_store_entry('store_crypto',
                                                           'p11_crypto')
        kmip_secret_store = self._get_secret_store_entry('kmip_plugin', None)
        db_secret_store = self._get_secret_store_entry('store_crypto',
                                                       'simple_crypto')

        project1 = self._create_project()
        project2 = self._create_project()
        project3 = self._create_project()

        # For project1 , make pkcs11 as preferred secret store
        self._create_project_store(project1.id, pkcs11_secret_store.id)
        # For project2 , make kmip as preferred secret store
        self._create_project_store(project2.id, kmip_secret_store.id)
        # For project3 , make db backend as preferred secret store
        self._create_project_store(project3.id, db_secret_store.id)

        keySpec = secret_store.KeySpec(secret_store.KeyAlgorithm.AES, 128)
        cm_manager = cm.get_manager()

        # For project1, verify store and crypto plugin instance used are pkcs11
        # specific
        plugin_found = manager.get_plugin_store(keySpec,
                                                project_id=project1.id)
        self.assertIsInstance(plugin_found,
                              store_crypto.StoreCryptoAdapterPlugin)
        crypto_plugin = cm.get_manager().get_plugin_store_generate(
            base.PluginSupportTypes.ENCRYPT_DECRYPT, project_id=project1.id)
        self.assertIsInstance(crypto_plugin, p11_crypto.P11CryptoPlugin)

        # For project2, verify store plugin instance is kmip specific
        # and there is no crypto plugin instance
        plugin_found = manager.get_plugin_store(keySpec,
                                                project_id=project2.id)
        self.assertIsInstance(plugin_found, kss.KMIPSecretStore)

        self.assertRaises(
            base.CryptoPluginNotFound, cm_manager.get_plugin_store_generate,
            base.PluginSupportTypes.ENCRYPT_DECRYPT, project_id=project2.id)

        # For project3, verify store and crypto plugin instance used are db
        # backend specific
        plugin_found = manager.get_plugin_store(keySpec,
                                                project_id=project3.id)
        self.assertIsInstance(plugin_found,
                              store_crypto.StoreCryptoAdapterPlugin)
        crypto_plugin = cm.get_manager().get_plugin_store_generate(
            base.PluginSupportTypes.ENCRYPT_DECRYPT, project_id=project3.id)
        self.assertIsInstance(crypto_plugin, simple_crypto.SimpleCryptoPlugin)

        # Make sure for project with no preferred setting, uses global default
        project4 = self._create_project()
        plugin_found = manager.get_plugin_store(keySpec,
                                                project_id=project4.id)
        self.assertIsInstance(plugin_found,
                              dataset['default_store_class'])



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_log import versionutils
from oslo_policy import policy

from barbican.common.policies import base


deprecated_transport_key_get = policy.DeprecatedRule(
    name='transport_key:get',
    check_str='rule:all_users',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_transport_key_delete = policy.DeprecatedRule(
    name='transport_key:delete',
    check_str='rule:service_admin',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_transport_keys_get = policy.DeprecatedRule(
    name='transport_keys:get',
    check_str='rule:all_users',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_transport_keys_post = policy.DeprecatedRule(
    name='transport_keys:post',
    check_str='rule:service_admin',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    policy.DocumentedRuleDefault(
        name='transport_key:get',
        check_str='True:%(enforce_new_defaults)s and role:reader',
        scope_types=['project'],
        description='Get a specific transport key.',
        operations=[
            {
                'path': '/v1/transport_keys/{key-id}}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_transport_key_get
    ),
    policy.DocumentedRuleDefault(
        name='transport_key:delete',
        check_str='True:%(enforce_new_defaults)s and role:admin',
        scope_types=['project'],
        description='Delete a specific transport key.',
        operations=[
            {
                'path': '/v1/transport_keys/{key-id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_transport_key_delete
    ),
    policy.DocumentedRuleDefault(
        name='transport_keys:get',
        check_str='True:%(enforce_new_defaults)s and role:reader',
        scope_types=['project'],
        description='Get a list of all transport keys.',
        operations=[
            {
                'path': '/v1/transport_keys',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_transport_keys_get
    ),
    policy.DocumentedRuleDefault(
        name='transport_keys:post',
        check_str='True:%(enforce_new_defaults)s and role:admin',
        scope_types=['project'],
        description='Create a new transport key.',
        operations=[
            {
                'path': '/v1/transport_keys',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_transport_keys_post
    ),

]


def list_rules():
    return rules



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_utils import timeutils
from oslo_versionedobjects import base as object_base

from barbican.common import exception
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repo
from barbican.objects import base
from barbican.objects import container_acl_user
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class ContainerACL(base.BarbicanObject, base.BarbicanPersistentObject,
                   object_base.VersionedObjectDictCompat):
    fields = {
        'container_id': fields.StringField(),
        'operation': fields.StringField(),
        'project_access': fields.BooleanField(default=True),
        'acl_users': fields.ListOfObjectsField('ContainerACLUser',
                                               default=list()),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.ContainerACL
    db_repo = repo.get_container_acl_repository()
    synthetic_fields = ['acl_users']

    def _validate_fields(self, change_fields):

        msg = u._("Must supply non-None {0} argument for ContainerACL entry.")

        if change_fields.get('container_id') is None:
            raise exception.MissingArgumentError(msg.format("container_id"))

        if change_fields.get('operation') is None:
            raise exception.MissingArgumentError(msg.format("operation"))

    def _get_db_entity(self, user_ids=None):
        return self.db_model(user_ids=user_ids, check_exc=False)

    def create(self, session=None, user_ids=None):
        change_fields = self._get_changed_persistent_fields()
        self._validate_fields(change_fields)
        db_entity = self._get_db_entity(user_ids=user_ids)
        db_entity.update(change_fields)
        db_entity = self.db_repo.create_from(db_entity, session=session)
        self._from_db_object(db_entity)

    def delete(self, session):
        entity_id = self.id
        self.db_repo.delete_entity_by_id(
            entity_id=entity_id, external_project_id=None, session=session)

    @classmethod
    def get_by_container_id(cls, container_id, session=None):
        entities_db = cls.db_repo.get_by_container_id(container_id, session)
        entities = [cls()._from_db_object(entity_db)
                    for entity_db in entities_db]
        return entities

    @classmethod
    def create_or_replace_from(cls, container, container_acl,
                               user_ids=None, session=None):
        """Create or replace Secret and SecretACL

        :param container: an instance of Container object
        :param container_acl: an instance of ContainerACL object
        :param user_ids:
        :param session: a session to connect with database
        """
        session = cls.get_session(session)
        container.updated_at = timeutils.utcnow()
        container.save(session=session)
        container_acl.updated_at = timeutils.utcnow()
        if container_acl.id is None:
            container_acl.create(session=session, user_ids=user_ids)
        else:
            container_acl.save(session=session)
        cls._create_or_replace_acl_users(container_acl=container_acl,
                                         user_ids=user_ids, session=session)

    @classmethod
    def create_or_replace_from_model(cls, container, container_acl,
                                     user_ids=None, session=None):
        """Create or replace Secret and SecretACL

        :param container: an instance of Container model
        :param container_acl: an instance of ContainerACL object
        :param user_ids:
        :param session: a session to connect with database

        It is used during converting from model to OVO. It will be removed
        after Container resource is implemented OVO.
        """
        session = cls.get_session(session)
        container.updated_at = timeutils.utcnow()
        container.save(session=session)
        now = timeutils.utcnow()
        container_acl.updated_at = now
        if container_acl.id is None:
            container_acl.create(session=session, user_ids=user_ids)
        else:
            container_acl.save(session=session)
        cls._create_or_replace_acl_users(container_acl=container_acl,
                                         user_ids=user_ids, session=session)

    @classmethod
    def _create_or_replace_acl_users(cls, container_acl, user_ids,
                                     session=None):
        if user_ids is None:
            return

        user_ids = set(user_ids)

        now = timeutils.utcnow()
        session = session or cls.get_session(session)
        container_acl.updated_at = now

        for acl_user in container_acl.acl_users:
            if acl_user.user_id in user_ids:  # input user_id already exists
                acl_user.updated_at = now
                acl_user.save(session=session)
                user_ids.remove(acl_user.user_id)
            else:
                acl_user.delete(session=session)

        for user_id in user_ids:
            acl_user = container_acl_user.ContainerACLUser(
                acl_id=container_acl.id, user_id=user_id)
            acl_user.create(session=session)

        if container_acl.id:
            container_acl.save(session=session)
        else:
            container_acl.create(session=session)

    @classmethod
    def get_count(cls, container_id, session=None):
        query = cls.db_repo.get_count(container_id, session)
        return query

    @classmethod
    def delete_acls_for_container(cls, container, session=None):

        # TODO(namnh)
        # After Container resource is implemented, This function
        # will be updated source code being used.
        session = cls.get_session(session=session)
        for entity in container.container_acls:
            entity.delete(session=session)

    @classmethod
    def delete_acls_for_container_model(cls, container, session=None):
        """Delete ACLs in Container

        Used during converting Model to OVO, it will be removed in near future.
        :param container: instance of Container model
        :param session: connection to database
        """
        cls.db_repo.delete_acls_for_container(container, session)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock


from barbican.common import exception
from barbican.plugin.crypto import pkcs11
from barbican.tests import utils


class WhenTestingPKCS11(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingPKCS11, self).setUp()

        self.lib = mock.Mock()
        self.lib.C_Initialize.return_value = pkcs11.CKR_OK
        self.lib.C_Finalize.return_value = pkcs11.CKR_OK
        self.lib.C_GetSlotList.side_effect = self._get_slot_list
        self.lib.C_GetTokenInfo.side_effect = self._get_token_info
        self.lib.C_OpenSession.side_effect = self._open_session
        self.lib.C_CloseSession.return_value = pkcs11.CKR_OK
        self.lib.C_GetSessionInfo.side_effect = self._get_session_user
        self.lib.C_Login.return_value = pkcs11.CKR_OK
        self.lib.C_FindObjectsInit.return_value = pkcs11.CKR_OK
        self.lib.C_FindObjects.side_effect = self._find_objects_one
        self.lib.C_FindObjectsFinal.return_value = pkcs11.CKR_OK
        self.lib.C_GenerateKey.side_effect = self._generate_key
        self.lib.C_GenerateRandom.side_effect = self._generate_random
        self.lib.C_SeedRandom.return_value = pkcs11.CKR_OK
        self.lib.C_EncryptInit.return_value = pkcs11.CKR_OK
        self.lib.C_Encrypt.side_effect = self._encrypt
        self.lib.C_DecryptInit.return_value = pkcs11.CKR_OK
        self.lib.C_Decrypt.side_effect = self._decrypt
        self.lib.C_WrapKey.side_effect = self._wrap_key
        self.lib.C_UnwrapKey.side_effect = self._unwrap_key
        self.lib.C_SignInit.return_value = pkcs11.CKR_OK
        self.lib.C_Sign.side_effect = self._sign
        self.lib.C_VerifyInit.return_value = pkcs11.CKR_OK
        self.lib.C_Verify.side_effect = self._verify
        self.lib.C_DestroyObject.return_value = pkcs11.CKR_OK
        self.ffi = pkcs11.build_ffi()
        setattr(self.ffi, 'dlopen', lambda x: self.lib)

        self.cfg_mock = mock.MagicMock(name='config mock')
        self.cfg_mock.library_path = '/dev/null'
        self.cfg_mock.login_passphrase = 'foobar'
        self.cfg_mock.rw_session = False
        self.cfg_mock.slot_id = 1
        self.cfg_mock.encryption_mechanism = 'CKM_AES_CBC'
        self.cfg_mock.hmac_mechanism = 'CKM_SHA256_HMAC'
        self.cfg_mock.key_wrap_mechanism = 'CKM_AES_KEY_WRAP_KWP'

        self.token_mock = mock.MagicMock()
        self.token_mock.label = b'myLabel'
        self.token_mock.serial_number = b'111111'

        self.pkcs11 = pkcs11.PKCS11(
            self.cfg_mock.library_path,
            self.cfg_mock.login_passphrase,
            slot_id=self.cfg_mock.slot_id,
            rw_session=self.cfg_mock.rw_session,
            encryption_mechanism=self.cfg_mock.encryption_mechanism,
            hmac_mechanism=self.cfg_mock.hmac_mechanism,
            key_wrap_mechanism=self.cfg_mock.key_wrap_mechanism,
            ffi=self.ffi,
        )

    def _generate_random(self, session, buf, length):
        self.ffi.buffer(buf)[:] = b'0' * length
        return pkcs11.CKR_OK

    def _get_slot_list(self, token_present, slot_ids_ptr, slots_ptr):
        # default to mocking only one slot (ID: 1)
        if slot_ids_ptr is not self.ffi.NULL:
            slot_ids_ptr[0] = 1
        slots_ptr[0] = 1
        return pkcs11.CKR_OK

    def _get_token_info(self, id, token_info_ptr):
        token_info_ptr.serialNumber = self.token_mock.serial_number
        token_info_ptr.label = self.token_mock.label
        return pkcs11.CKR_OK

    def _get_two_slot_list(self, token_present, slot_ids_ptr, slots_ptr):
        # mock two slots (IDs: 1, 2)
        if slot_ids_ptr is not self.ffi.NULL:
            slot_ids_ptr[0] = 1
            slot_ids_ptr[1] = 2
        slots_ptr[0] = 2
        return pkcs11.CKR_OK

    def _get_two_token_info_same_label(self, id, token_info_ptr):
        token_info_ptr.serialNumber = (str(id) * 6).encode('UTF-8')
        token_info_ptr.label = self.token_mock.label
        return pkcs11.CKR_OK

    def _get_session_public(self, session, session_info_ptr):
        if self.cfg_mock.rw_session:
            session_info_ptr[0].state = pkcs11.CKS_RW_PUBLIC_SESSION
        else:
            session_info_ptr[0].state = pkcs11.CKS_RO_PUBLIC_SESSION
        return pkcs11.CKR_OK

    def _get_session_user(self, session, session_info_ptr):
        if self.cfg_mock.rw_session:
            session_info_ptr[0].state = pkcs11.CKS_RW_USER_FUNCTIONS
        else:
            session_info_ptr[0].state = pkcs11.CKS_RO_USER_FUNCTIONS
        return pkcs11.CKR_OK

    def _open_session(self, *args, **kwargs):
        args[4][0] = int(1)
        return pkcs11.CKR_OK

    def _find_objects_one(self, session, obj_handle_ptr, max_count, count):
        obj_handle_ptr[0] = int(2)
        count[0] = 1
        return pkcs11.CKR_OK

    def _find_objects_two(self, session, obj_handle_ptr, max_count, count):
        obj_handle_ptr[0] = int(2)
        count[0] = 2
        return pkcs11.CKR_OK

    def _find_objects_zero(self, session, obj_handle_ptr, max_count, count):
        count[0] = 0
        return pkcs11.CKR_OK

    def _generate_key(self, session, mech, attributes, attributes_len,
                      obj_handle_ptr):
        obj_handle_ptr[0] = int(3)
        return pkcs11.CKR_OK

    def _encrypt(self, session, pt, pt_len, ct, ct_len):
        if self.pkcs11.encrypt_gen_iv:
            self.ffi.buffer(ct)[:] = pt[::-1] + b'0' * self.pkcs11.gcmtagsize
        else:
            self.ffi.buffer(ct)[:] = pt[::-1] + b'0' * (self.pkcs11.gcmtagsize
                                                        * 2)
        return pkcs11.CKR_OK

    def _decrypt(self, session, ct, ct_len, pt, pt_len):
        tmp = ct[:-self.pkcs11.gcmtagsize][::-1]
        self.ffi.buffer(pt)[:len(tmp)] = tmp
        return pkcs11.CKR_OK

    def _wrap_key(self, *args, **kwargs):
        wrapped_key = args[4]
        wrapped_key_len = args[5]
        wrapped_key_len[0] = int(16)
        if wrapped_key != self.ffi.NULL:
            self.ffi.buffer(wrapped_key)[:] = b'0' * 16
        return pkcs11.CKR_OK

    def _unwrap_key(self, *args, **kwargs):
        unwrapped_key = args[7]
        unwrapped_key[0] = int(1)
        return pkcs11.CKR_OK

    def _sign(self, *args, **kwargs):
        buf = args[3]
        buf_len = args[4]
        self.ffi.buffer(buf)[:] = b'0' * buf_len[0]
        return pkcs11.CKR_OK

    def _verify(self, *args, **kwargs):
        return pkcs11.CKR_OK

    def test_init_raises_invalid_encryption_mechanism(self):
        self.assertRaises(
            ValueError,
            pkcs11.PKCS11,
            self.cfg_mock.library_path,
            self.cfg_mock.login_passphrase,
            encryption_mechanism='CKM_BOGUS')

    def test_init_raises_invalid_hmac_mechanism(self):
        self.assertRaises(
            ValueError,
            pkcs11.PKCS11,
            self.cfg_mock.library_path,
            self.cfg_mock.login_passphrase,
            encryption_mechanism='CKM_AES_GCM',
            hmac_mechanism='CKM_BOGUS')

    def test_init_raises_invalid_key_wrap_mechanism(self):
        self.assertRaises(
            ValueError,
            pkcs11.PKCS11,
            self.cfg_mock.library_path,
            self.cfg_mock.login_passphrase,
            encryption_mechanism='CKM_AES_GCM',
            hmac_mechanism='CKM_SHA256_HMAC',
            key_wrap_mechanism='CKM_BOGUS')

    def test_get_slot_id_from_serial_number(self):
        slot_id = self.pkcs11._get_slot_id('111111', None, 2)
        self.assertEqual(1, slot_id)

    def test_get_slot_id_from_label(self):
        slot_id = self.pkcs11._get_slot_id(None, ['myLabel'], 2)
        self.assertEqual(1, slot_id)

    def test_get_slot_id_backwards_compatibility(self):
        slot_id = self.pkcs11._get_slot_id(None, None, 5)
        self.assertEqual(5, slot_id)

    def test_get_slot_id_from_serial_ignores_label(self):
        slot_id = self.pkcs11._get_slot_id('111111', ['badLabel'], 2)
        self.assertEqual(1, slot_id)

    def test_get_slot_id_from_serial_ignores_given_slot(self):
        slot_id = self.pkcs11._get_slot_id('111111', None, 3)
        self.assertEqual(1, slot_id)

    def test_get_slot_id_from_label_ignores_given_slot(self):
        slot_id = self.pkcs11._get_slot_id(None, ['myLabel'], 3)
        self.assertEqual(1, slot_id)

    def test_get_slot_id_serial_not_found(self):
        self.assertRaises(ValueError,
                          self.pkcs11._get_slot_id, '222222', None, 1)

    def test_get_slot_id_label_not_found(self):
        self.assertRaises(ValueError,
                          self.pkcs11._get_slot_id, None, ['myLabelbad'], 1)

    def test_get_slot_id_two_tokens_same_label(self):
        self.lib.C_GetSlotList.side_effect = self._get_two_slot_list
        self.lib.C_GetTokenInfo.side_effect = \
            self._get_two_token_info_same_label
        slot_id = self.pkcs11._get_slot_id(None, ['myLabel'], 3)
        self.assertEqual(1, slot_id)

    def test_public_get_session(self):
        self.lib.C_GetSessionInfo.side_effect = self._get_session_public
        sess = self.pkcs11.get_session()

        self.assertEqual(1, sess)

        self.assertEqual(2, self.lib.C_OpenSession.call_count)
        self.assertEqual(2, self.lib.C_GetSessionInfo.call_count)
        self.assertEqual(1, self.lib.C_Login.call_count)
        self.assertEqual(1, self.lib.C_CloseSession.call_count)

    def test_user_get_session(self):
        self.pkcs11.get_session()

        self.assertEqual(2, self.lib.C_OpenSession.call_count)
        self.assertEqual(2, self.lib.C_GetSessionInfo.call_count)
        self.assertEqual(0, self.lib.C_Login.call_count)

    def test_seed_random(self):
        rd = "random-data"
        session = 'session'
        self.pkcs11._seed_random(session, rd)
        self.lib.C_SeedRandom.assert_called_once_with(
            session, mock.ANY, len(rd))

    def test_generate_random(self):
        r = self.pkcs11.generate_random(32, mock.MagicMock())

        self.assertEqual(b'0' * 32, r)

        self.assertEqual(2, self.lib.C_GenerateRandom.call_count)

    def test_rng_self_test_fail(self):
        def _bad_generate_random(session, buf, length):
            self.ffi.buffer(buf)[:] = b'\x00' * length
            return pkcs11.CKR_OK
        self.lib.C_GenerateRandom.side_effect = _bad_generate_random
        self.assertRaises(exception.P11CryptoPluginException,
                          self.pkcs11._rng_self_test, mock.MagicMock())

    def test_get_key_handle_one_key(self):
        key = self.pkcs11.get_key_handle('CKK_AES', 'foo', mock.MagicMock())

        self.assertEqual(2, key)

        self.assertEqual(1, self.lib.C_FindObjectsInit.call_count)
        self.assertEqual(1, self.lib.C_FindObjects.call_count)
        self.assertEqual(1, self.lib.C_FindObjectsFinal.call_count)

    def test_get_key_handle_no_keys(self):
        self.lib.C_FindObjects.side_effect = self._find_objects_zero
        key = self.pkcs11.get_key_handle('CKK_AES', 'foo', mock.MagicMock())

        self.assertIsNone(key)

        self.assertEqual(1, self.lib.C_FindObjectsInit.call_count)
        self.assertEqual(1, self.lib.C_FindObjects.call_count)
        self.assertEqual(1, self.lib.C_FindObjectsFinal.call_count)

    def test_get_key_handle_multiple_keys(self):
        self.lib.C_FindObjects.side_effect = self._find_objects_two

        self.assertRaises(exception.P11CryptoPluginKeyException,
                          self.pkcs11.get_key_handle, 'CKK_AES', 'foo',
                          mock.MagicMock())

        self.assertEqual(1, self.lib.C_FindObjectsInit.call_count)
        self.assertEqual(1, self.lib.C_FindObjects.call_count)
        self.assertEqual(1, self.lib.C_FindObjectsFinal.call_count)

    def test_generate_session_key(self):
        key = self.pkcs11.generate_key('CKK_AES', 16, 'CKM_AES_KEY_GEN',
                                       mock.MagicMock(), encrypt=True)

        self.assertEqual(3, key)

        self.assertEqual(1, self.lib.C_GenerateKey.call_count)

    def test_generate_master_key(self):
        key = self.pkcs11.generate_key('CKK_AES', 16, 'CKM_AES_KEY_GEN',
                                       mock.MagicMock(), key_label='key',
                                       encrypt=True, master_key=True)

        self.assertEqual(3, key)

        self.assertEqual(1, self.lib.C_GenerateKey.call_count)

    def test_generate_key_no_flags(self):
        self.assertRaises(exception.P11CryptoPluginException,
                          self.pkcs11.generate_key, 'CKK_AES', 16,
                          mock.MagicMock(), mock.MagicMock())

    def test_generate_master_key_no_label(self):
        self.assertRaises(ValueError, self.pkcs11.generate_key,
                          'CKK_AES', 16,
                          mock.MagicMock(), mock.MagicMock(),
                          encrypt=True, master_key=True)

    def test_encrypt_with_no_iv_generation(self):
        pt = b'0123456789ABCDEF'
        self.pkcs11.encrypt_gen_iv = False
        ct = self.pkcs11._VENDOR_SAFENET_CKM_AES_GCM_encrypt(
            mock.MagicMock(),
            pt, mock.MagicMock()
        )

        self.assertEqual(ct['ct'][:len(pt)], pt[::-1])
        self.assertGreater(len(ct['iv']), 0)

        self.assertEqual(1, self.lib.C_GenerateRandom.call_count)
        self.assertEqual(1, self.lib.C_EncryptInit.call_count)
        self.assertEqual(1, self.lib.C_Encrypt.call_count)

    def test_encrypt_with_iv_generation(self):
        pt = b'0123456789ABCDEF'
        self.pkcs11.encrypt_gen_iv = True
        ct = self.pkcs11._VENDOR_SAFENET_CKM_AES_GCM_encrypt(
            mock.MagicMock(), pt, mock.MagicMock()
        )

        self.assertEqual(ct['ct'][:len(pt)], pt[::-1])
        self.assertGreater(len(ct['iv']), 0)

        self.assertEqual(2, self.lib.C_GenerateRandom.call_count)
        self.assertEqual(1, self.lib.C_EncryptInit.call_count)
        self.assertEqual(1, self.lib.C_Encrypt.call_count)

    def test_decrypt(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.noncesize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize
        self.assertEqual(pt[:pt_len], ct[:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_decrypt_with_pad(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.blocksize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize - 3
        self.assertEqual(pt[:pt_len], ct[3:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_decrypt_with_pad_new_iv(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.gcmtagsize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize
        self.assertEqual(pt[:pt_len], ct[:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_decrypt_with_pad_wrong_size(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.blocksize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize
        self.assertEqual(pt[:pt_len], ct[:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_decrypt_with_pad_wrong_length(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.blocksize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize
        self.assertEqual(pt[:pt_len], ct[:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_decrypt_with_too_large_pad(self):
        ct = b'c2VjcmV0a2V5BwcHBwcHBw=='
        iv = b'0' * self.pkcs11.blocksize
        pt = self.pkcs11.decrypt('VENDOR_SAFENET_CKM_AES_GCM',
                                 mock.MagicMock(), iv, ct, mock.MagicMock())

        pt_len = len(ct) - self.pkcs11.gcmtagsize
        self.assertEqual(pt[:pt_len], ct[:-self.pkcs11.gcmtagsize][::-1])

        self.assertEqual(1, self.lib.C_DecryptInit.call_count)
        self.assertEqual(1, self.lib.C_Decrypt.call_count)

    def test_wrap_key_with_iv_generation(self):
        self.pkcs11.key_wrap_gen_iv = True
        wkek = self.pkcs11.wrap_key(mock.Mock(), mock.Mock(), mock.Mock())
        self.assertGreater(len(wkek['iv']), 0)
        self.assertEqual(b'0' * 16, wkek['wrapped_key'])

        self.assertEqual(2, self.lib.C_GenerateRandom.call_count)
        self.assertEqual(2, self.lib.C_WrapKey.call_count)

    def test_wrap_key_no_iv_generation(self):
        self.pkcs11.key_wrap_gen_iv = False
        wkek = self.pkcs11.wrap_key(mock.Mock(), mock.Mock(), mock.Mock())
        self.assertIsNone(wkek['iv'])
        self.assertEqual(b'0' * 16, wkek['wrapped_key'])

        self.assertEqual(1, self.lib.C_GenerateRandom.call_count)
        self.assertEqual(2, self.lib.C_WrapKey.call_count)

    def test_unwrap_key(self):
        kek = self.pkcs11.unwrap_key(
            'CKM_AES_CBC_PAD',
            b'0' * 16,
            b'0' * 16,
            b'0' * 16,
            mock.Mock())
        self.assertEqual(1, kek)

        self.assertEqual(self.lib.C_UnwrapKey.call_count, 1)

    def test_compute_hmac(self):
        buf = self.pkcs11.compute_hmac(mock.MagicMock(), mock.MagicMock(),
                                       mock.MagicMock())
        self.assertEqual(32, len(buf))

        self.assertEqual(1, self.lib.C_SignInit.call_count)
        self.assertEqual(1, self.lib.C_Sign.call_count)

    def test_verify_hmac(self):
        self.pkcs11.verify_hmac(mock.MagicMock(), mock.MagicMock(),
                                mock.MagicMock(), mock.MagicMock())

        self.assertEqual(1, self.lib.C_VerifyInit.call_count)
        self.assertEqual(1, self.lib.C_Verify.call_count)

    def test_destroy_object(self):
        self.pkcs11.destroy_object(mock.MagicMock(), mock.MagicMock())

        self.assertEqual(1, self.lib.C_DestroyObject.call_count)

    def test_invalid_build_attributes(self):
        self.assertRaises(TypeError, self.pkcs11._build_attributes,
                          [pkcs11.Attribute(pkcs11.CKA_CLASS, {})])

    def test_finalize(self):
        self.pkcs11.finalize()

        self.assertEqual(1, self.lib.C_Finalize.call_count)

    def test_finalize_ignores_trustway_network_errors(self):
        self.lib.C_Finalize.return_value = 0x81000071
        self.pkcs11.finalize()

        self.assertEqual(1, self.lib.C_Finalize.call_count)

    def test_check_error(self):
        self.assertIsNone(self.pkcs11._check_error(pkcs11.CKR_OK))

    def test_check_error_with_without_specific_handling(self):
        self.assertRaises(exception.P11CryptoPluginException,
                          self.pkcs11._check_error, 5)

    def test_check_error_with_token_error(self):
        self.assertRaises(exception.P11CryptoTokenException,
                          self.pkcs11._check_error, 0xe0)

    def test_converting_unicode_to_bytes(self):
        self.assertEqual(b'foo', pkcs11._to_bytes('foo'))

    def test_converting_default_str_type_to_bytes(self):
        self.assertEqual(b'foo', pkcs11._to_bytes('foo'))



# Copyright (c) 2014 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
An implementation of the SecretStore that uses the KMIP backend.
"""

import base64
import os
import ssl
import stat

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization

from kmip.core import enums
from kmip.core.factories import credentials
from kmip.pie import client
from kmip.pie import objects

from oslo_config import cfg
from oslo_log import log

from barbican.common import config
from barbican.common import exception
from barbican import i18n as u  # noqa
from barbican.plugin.interface import secret_store as ss
from barbican.plugin.util import translations

LOG = log.getLogger(__name__)

CONF = config.new_config()

kmip_opt_group = cfg.OptGroup(name='kmip_plugin', title='KMIP Plugin')
kmip_opts = [
    cfg.StrOpt('username',
               help=u._('Username for authenticating with KMIP server')
               ),
    cfg.StrOpt('password',
               help=u._('Password for authenticating with KMIP server'),
               secret=True,
               ),
    cfg.StrOpt('host',
               default='localhost',
               help=u._('Address of the KMIP server')
               ),
    cfg.PortOpt('port',
                default=5696,
                help=u._('Port for the KMIP server'),
                ),
    cfg.StrOpt('ssl_version',
               default='PROTOCOL_TLSv1_2',
               help=u._('SSL version, maps to the module ssl\'s constants'),
               ),
    cfg.StrOpt('ca_certs',
               help=u._('File path to concatenated "certification authority" '
                        'certificates'),
               ),
    cfg.StrOpt('certfile',
               help=u._('File path to local client certificate'),
               ),
    cfg.StrOpt('keyfile',
               help=u._('File path to local client certificate keyfile'),
               ),
    cfg.BoolOpt('pkcs1_only',
                default=False,
                help=u._('Only support PKCS#1 encoding of asymmetric keys'),
                ),
    cfg.StrOpt('plugin_name',
               help=u._('User friendly plugin name'),
               default='KMIP HSM'),
]
CONF.register_group(kmip_opt_group)
CONF.register_opts(kmip_opts, group=kmip_opt_group)
config.parse_args(CONF)


def list_opts():
    yield kmip_opt_group, kmip_opts


attribute_debug_msg = "Created attribute type %s with value %s"


def convert_pem_to_der(pem_pkcs1):
    # cryptography adds an extra '\n' to end of PEM file
    # added if statement so if future version removes extra \n tests will not
    # break
    if pem_pkcs1.endswith(b'\n'):
        pem_pkcs1 = pem_pkcs1[:-1]
    # neither PyCrypto or cryptography support export in DER format with PKCS1
    # encoding so doing by hand
    der_pkcs1_b64 = b''.join(pem_pkcs1.split(b'\n')[1:-1])
    der_pkcs1 = base64.b64decode(der_pkcs1_b64)
    return der_pkcs1


def get_public_key_der_pkcs1(pem):
    """Converts PEM public key to DER PKCS1"""
    rsa_public = serialization.load_pem_public_key(
        pem,
        backend=default_backend())
    pem_pkcs1 = rsa_public.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.PKCS1)
    return convert_pem_to_der(pem_pkcs1)


def get_private_key_der_pkcs1(pem):
    """Converts PEM private key to DER PKCS1"""
    rsa_private = serialization.load_pem_private_key(
        pem,
        None,
        backend=default_backend())
    pem_pkcs1 = rsa_private.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption())
    return convert_pem_to_der(pem_pkcs1)


class KMIPSecretStoreError(exception.BarbicanException):
    def __init__(self, message):
        super(KMIPSecretStoreError, self).__init__(message)


class KMIPSecretStoreActionNotSupported(exception.BarbicanHTTPException):
    """Raised if no plugins are found that support the requested operation."""

    client_message = u._("KMIP plugin action not support.")
    status_code = 400

    def __init__(self, message):
        self.message = message
        super(KMIPSecretStoreActionNotSupported, self).__init__()


class KMIPSecretStore(ss.SecretStoreBase):

    KEY_UUID = "key_uuid"
    VALID_BIT_LENGTHS = "valid_bit_lengths"
    KMIP_ALGORITHM_ENUM = "kmip_algorithm_enum"

    def __init__(self, conf=CONF):
        """Initializes KMIPSecretStore

        Creates a dictionary of mappings between SecretStore enum values
        and pyKMIP enum values. Initializes the KMIP client with credentials
        needed to connect to the KMIP server.
        """
        super(KMIPSecretStore, self).__init__()
        self.valid_alg_dict = {
            ss.KeyAlgorithm.AES: {
                KMIPSecretStore.VALID_BIT_LENGTHS: [128, 192, 256],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.AES},
            ss.KeyAlgorithm.DES: {
                KMIPSecretStore.VALID_BIT_LENGTHS: [56],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.DES},
            ss.KeyAlgorithm.DESEDE: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [56, 64, 112, 128, 168, 192],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.TRIPLE_DES},
            ss.KeyAlgorithm.DSA: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [1024, 2048, 3072],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.DSA},
            ss.KeyAlgorithm.HMACSHA1: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.HMAC_SHA1},
            ss.KeyAlgorithm.HMACSHA256: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.HMAC_SHA256},
            ss.KeyAlgorithm.HMACSHA384: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.HMAC_SHA384},
            ss.KeyAlgorithm.HMACSHA512: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.HMAC_SHA512},
            ss.KeyAlgorithm.RSA: {
                KMIPSecretStore.VALID_BIT_LENGTHS:
                [1024, 2048, 3072, 4096],
                KMIPSecretStore.KMIP_ALGORITHM_ENUM:
                enums.CryptographicAlgorithm.RSA},
        }
        self.pkcs1_only = conf.kmip_plugin.pkcs1_only
        if self.pkcs1_only:
            LOG.debug("KMIP secret store only supports PKCS#1")
            del self.valid_alg_dict[ss.KeyAlgorithm.DSA]
        self.kmip_barbican_alg_map = {
            enums.CryptographicAlgorithm.AES: ss.KeyAlgorithm.AES,
            enums.CryptographicAlgorithm.DES: ss.KeyAlgorithm.DES,
            enums.CryptographicAlgorithm.TRIPLE_DES: ss.KeyAlgorithm.DESEDE,
            enums.CryptographicAlgorithm.DSA: ss.KeyAlgorithm.DSA,
            enums.CryptographicAlgorithm.HMAC_SHA1: ss.KeyAlgorithm.HMACSHA1,
            enums.CryptographicAlgorithm.HMAC_SHA256:
                ss.KeyAlgorithm.HMACSHA256,
            enums.CryptographicAlgorithm.HMAC_SHA384:
                ss.KeyAlgorithm.HMACSHA384,
            enums.CryptographicAlgorithm.HMAC_SHA512:
                ss.KeyAlgorithm.HMACSHA512,
            enums.CryptographicAlgorithm.RSA: ss.KeyAlgorithm.RSA
        }

        self.plugin_name = conf.kmip_plugin.plugin_name

        if conf.kmip_plugin.keyfile is not None:
            self._validate_keyfile_permissions(conf.kmip_plugin.keyfile)

        if (conf.kmip_plugin.username is None) and (
                conf.kmip_plugin.password is None):
            self.credential = None
        else:
            credential_type = enums.CredentialType.USERNAME_AND_PASSWORD
            credential_value = {'Username': conf.kmip_plugin.username,
                                'Password': conf.kmip_plugin.password}
            self.credential = (
                credentials.CredentialFactory().create_credential(
                    credential_type,
                    credential_value))

        config = conf.kmip_plugin

        if not getattr(ssl, config.ssl_version, None):
            LOG.error("The configured SSL version (%s) is not available"
                      " on the system.", config.ssl_version)

        self.client = client.ProxyKmipClient(
            hostname=config.host,
            port=config.port,
            cert=config.certfile,
            key=config.keyfile,
            ca=config.ca_certs,
            ssl_version=config.ssl_version,
            username=config.username,
            password=config.password)

    def get_plugin_name(self):
        return self.plugin_name

    def generate_symmetric_key(self, key_spec):
        """Generate a symmetric key.

        Creates KMIP attribute objects based on the given KeySpec to send to
        the server.

        :param key_spec: KeySpec with symmetric algorithm and bit_length
        :returns: dictionary holding key_id returned by server
        :raises: SecretGeneralException, SecretAlgorithmNotSupportedException
        """
        LOG.debug("Starting symmetric key generation with KMIP plugin")
        if not self.generate_supports(key_spec):
            raise ss.SecretAlgorithmNotSupportedException(
                key_spec.alg)

        if key_spec.alg.lower() not in ss.KeyAlgorithm.SYMMETRIC_ALGORITHMS:
            raise KMIPSecretStoreError(
                u._("An unsupported algorithm {algorithm} was passed to the "
                    "'generate_symmetric_key' method").format(
                        algorithm=key_spec.alg))

        algorithm = self._get_kmip_algorithm(key_spec.alg.lower())
        try:
            with self.client:
                LOG.debug("Opened connection to KMIP client for secret "
                          "generation")
                uuid = self.client.create(algorithm, key_spec.bit_length)
                LOG.debug("SUCCESS: Symmetric key generated with "
                          "uuid: %s", uuid)
                return {KMIPSecretStore.KEY_UUID: uuid}
        except Exception as e:
            LOG.exception("Error opening or writing to client")
            raise ss.SecretGeneralException(e)

    def generate_asymmetric_key(self, key_spec):
        """Generate an asymmetric key pair.

        Creates KMIP attribute objects based on the given KeySpec to send to
        the server. The KMIP Secret Store currently does not support
        protecting the private key with a passphrase.

        :param key_spec: KeySpec with asymmetric algorithm and bit_length
        :returns: AsymmetricKeyMetadataDTO with the key UUIDs
        :raises: SecretGeneralException, SecretAlgorithmNotSupportedException
                 KMIPSecretStoreActionNotSupported
        """
        LOG.debug("Starting asymmetric key generation with KMIP plugin")
        if not self.generate_supports(key_spec):
            raise ss.SecretAlgorithmNotSupportedException(
                key_spec.alg)

        if key_spec.alg.lower() not in ss.KeyAlgorithm.ASYMMETRIC_ALGORITHMS:
            raise ss.SecretAlgorithmNotSupportedException(key_spec.alg)

        if key_spec.passphrase:
            raise KMIPSecretStoreActionNotSupported(
                u._('KMIP plugin does not currently support protecting the '
                    'private key with a passphrase'))

        algorithm = self._get_kmip_algorithm(key_spec.alg.lower())
        length = key_spec.bit_length

        try:
            with self.client:
                LOG.debug("Opened connection to KMIP client for "
                          "asymmetric secret generation")
                public_uuid, private_uuid = self.client.create_key_pair(
                    algorithm, length)
                LOG.debug("SUCCESS: Asymmetric key pair generated with "
                          "public key uuid: %(public_uuid)s and "
                          "private key uuid: %(private_uuid)s" %
                          {'public_uuid': public_uuid,
                           'private_uuid': private_uuid})
                private_key_metadata = {KMIPSecretStore.KEY_UUID: private_uuid}
                public_key_metadata = {KMIPSecretStore.KEY_UUID: public_uuid}
                passphrase_metadata = None
                return ss.AsymmetricKeyMetadataDTO(private_key_metadata,
                                                   public_key_metadata,
                                                   passphrase_metadata)
        except Exception as e:
            LOG.exception("Error opening or writing to client")
            raise ss.SecretGeneralException(e)

    def store_secret(self, secret_dto):
        """Stores a secret

        To store a secret in KMIP, the attributes must be known.

        :param secret_dto: SecretDTO of the secret to be stored
        :returns: Dictionary holding the key_uuid assigned by KMIP
        :raises: SecretGeneralException, SecretAlgorithmNotSupportedException
        """
        LOG.debug("Starting secret storage with KMIP plugin")
        if not self.store_secret_supports(secret_dto.key_spec):
            raise ss.SecretAlgorithmNotSupportedException(
                secret_dto.key_spec.alg)

        secret_type = secret_dto.type
        object_type, key_format_type = (
            self._map_type_ss_to_kmip(secret_type))
        if object_type is None:
            raise KMIPSecretStoreError(
                u._('Secret object type {object_type} is '
                    'not supported').format(object_type=object_type))

        secret = self._get_kmip_secret(secret_dto)

        try:
            with self.client:
                LOG.debug("Opened connection to KMIP client")
                uuid = self.client.register(secret)
                LOG.debug("SUCCESS: Key stored with uuid: %s", uuid)
                return {KMIPSecretStore.KEY_UUID: uuid}
        except Exception as e:
            LOG.exception("Error opening or writing to client")
            raise ss.SecretGeneralException(e)

    def get_secret(self, secret_type, secret_metadata):
        """Gets a secret

        :param secret_type: secret type
        :param secret_metadata: Dictionary of key metadata, requires:
        {'key_uuid': <uuid of key>}
        :returns: SecretDTO of the retrieved Secret
        :raises: SecretGeneralException
        """
        LOG.debug("Starting secret retrieval with KMIP plugin")
        uuid = str(secret_metadata[KMIPSecretStore.KEY_UUID])
        try:
            with self.client:
                LOG.debug("Opened connection to KMIP client for secret "
                          "retrieval")
                managed_object = self.client.get(uuid)
                return self._get_barbican_secret(managed_object, secret_type)
        except Exception as e:
            LOG.exception("Error opening or writing to client")
            raise ss.SecretGeneralException(e)

    def generate_supports(self, key_spec):
        """Key generation supported?

        Specifies whether the plugin supports key generation with the
        given key_spec. Currently, asymmetric key pair generation does not
        support encrypting the private key with a passphrase.

        Checks both the algorithm and the bit length. Only symmetric
        algorithms are currently supported.
        :param key_spec: KeySpec for secret to be generates
        :returns: boolean indicating if secret can be generated
        """
        alg_dict_entry = self.valid_alg_dict.get(key_spec.alg.lower())
        if alg_dict_entry:
            valid_bit_lengths = alg_dict_entry.get(
                KMIPSecretStore.VALID_BIT_LENGTHS)
            if (key_spec.bit_length in valid_bit_lengths
                    or not valid_bit_lengths):
                return True
        return False

    def delete_secret(self, secret_metadata):
        """Deletes the secret whose metadata is included in the dictionary.

        Returns nothing if successful, raises an exception if an error occurs
        :param secret_metadata: Dictionary of key metadata, requires:
        {'key_uuid': <uuid of key>}
        :raises: SecretGeneralException
        """
        LOG.debug("Starting secret deletion with KMIP plugin")
        uuid = str(secret_metadata[KMIPSecretStore.KEY_UUID])
        try:
            with self.client:
                LOG.debug("Opened connection to KMIP client")
                self.client.destroy(uuid)
        except Exception as e:
            LOG.exception("Error opening or writing to client")
            raise ss.SecretGeneralException(e)

    def store_secret_supports(self, key_spec):
        """Key storage supported?

        Specifies whether the plugin supports storage of the secret given
        the attributes included in the KeySpec.

        For now this always returns true if the key spec's algorithm and bit
        length are not specified. The secret type may need to be added in the
        future. This must always return true if the algorithm and bit length
        are not specified because some secret types, like certificate, do not
        require algorithm and bit length, so true must always be returned for
        those cases.
        :param key_spec: KeySpec of secret to be stored
        :returns: boolean indicating if secret can be stored
        """
        if key_spec.alg is not None and key_spec.bit_length is not None:
            return self.generate_supports(key_spec)
        else:
            return True

    def _get_kmip_secret(self, secret_dto):
        """Builds a KMIP object from a SecretDTO

        This is needed for register calls. The Barbican object needs to be
        converted to KMIP object before it can be stored

        :param secret_dto: SecretDTO of secret to be stored
        :returns: KMIP object
        """
        secret_type = secret_dto.type
        key_spec = secret_dto.key_spec
        object_type, key_format_type = (
            self._map_type_ss_to_kmip(secret_type))

        normalized_secret = self._normalize_secret(secret_dto.secret,
                                                   secret_type)
        kmip_object = None
        if object_type == enums.ObjectType.CERTIFICATE:
            kmip_object = objects.X509Certificate(normalized_secret)
        elif object_type == enums.ObjectType.OPAQUE_DATA:
            opaque_type = enums.OpaqueDataType.NONE
            kmip_object = objects.OpaqueObject(normalized_secret,
                                               opaque_type)
        elif object_type == enums.ObjectType.PRIVATE_KEY:
            algorithm = self._get_kmip_algorithm(key_spec.alg)
            length = key_spec.bit_length
            format_type = enums.KeyFormatType.PKCS_8
            kmip_object = objects.PrivateKey(
                algorithm, length, normalized_secret, format_type)
        elif object_type == enums.ObjectType.PUBLIC_KEY:
            algorithm = self._get_kmip_algorithm(key_spec.alg)
            length = key_spec.bit_length
            format_type = enums.KeyFormatType.X_509
            kmip_object = objects.PublicKey(
                algorithm, length, normalized_secret, format_type)
        elif object_type == enums.ObjectType.SYMMETRIC_KEY:
            algorithm = self._get_kmip_algorithm(key_spec.alg)
            length = key_spec.bit_length
            kmip_object = objects.SymmetricKey(algorithm, length,
                                               normalized_secret)
        elif object_type == enums.ObjectType.SECRET_DATA:
            data_type = enums.SecretDataType.PASSWORD
            kmip_object = objects.SecretData(normalized_secret, data_type)

        return kmip_object

    def _get_kmip_algorithm(self, ss_algorithm):
        alg_entry = self.valid_alg_dict.get(ss_algorithm)
        return alg_entry.get(KMIPSecretStore.KMIP_ALGORITHM_ENUM)

    def _get_barbican_secret(self, managed_object, secret_type):
        object_type = managed_object.object_type
        secret = managed_object.value
        if (object_type == enums.ObjectType.SYMMETRIC_KEY or
                object_type == enums.ObjectType.PRIVATE_KEY or
                object_type == enums.ObjectType.PUBLIC_KEY):
            algorithm = self.kmip_barbican_alg_map[
                managed_object.cryptographic_algorithm]
            length = managed_object.cryptographic_length
            key_spec = ss.KeySpec(algorithm, length)
        else:
            key_spec = ss.KeySpec()

        secret = self._denormalize_secret(secret, secret_type)
        secret_dto = ss.SecretDTO(
            secret_type,
            secret,
            key_spec,
            content_type=None,
            transport_key=None)
        return secret_dto

    def _map_type_ss_to_kmip(self, object_type):
        """Map SecretType to KMIP type enum

        Returns None if the type is not supported.
        :param object_type: SecretType enum value
        :returns: KMIP type enums if supported, None if not supported
        """
        if object_type == ss.SecretType.SYMMETRIC:
            return enums.ObjectType.SYMMETRIC_KEY, enums.KeyFormatType.RAW
        elif object_type == ss.SecretType.PRIVATE:
            if self.pkcs1_only:
                return enums.ObjectType.PRIVATE_KEY, enums.KeyFormatType.PKCS_1
            else:
                return enums.ObjectType.PRIVATE_KEY, enums.KeyFormatType.PKCS_8
        elif object_type == ss.SecretType.PUBLIC:
            if self.pkcs1_only:
                return enums.ObjectType.PUBLIC_KEY, enums.KeyFormatType.PKCS_1
            else:
                return enums.ObjectType.PUBLIC_KEY, enums.KeyFormatType.X_509
        elif object_type == ss.SecretType.CERTIFICATE:
            return enums.ObjectType.CERTIFICATE, enums.KeyFormatType.X_509
        elif object_type == ss.SecretType.PASSPHRASE:
            return enums.ObjectType.SECRET_DATA, enums.KeyFormatType.RAW
        elif object_type == ss.SecretType.OPAQUE:
            return enums.ObjectType.OPAQUE_DATA, enums.KeyFormatType.RAW
        else:
            return None, None

    def _raise_secret_general_exception(self, result):
        msg = u._(
            "Status: {status}, Reason: {reason}, "
            "Message: {message}"
        ).format(
            status=result.result_status,
            reason=result.result_reason,
            message=result.result_message
        )
        LOG.error("ERROR from KMIP server: %s", msg)
        raise ss.SecretGeneralException(msg)

    def _validate_keyfile_permissions(self, path):
        """Check that file has permissions appropriate for a sensitive key

        Key files are extremely sensitive, they should be owned by the user
        who they relate to. They should be readable only (to avoid accidental
        changes). They should not be readable or writable by any other user.

        :raises: KMIPSecretStoreError
        """
        expected = (stat.S_IRUSR | stat.S_IFREG)  # 0o100400
        st = os.stat(path)
        if st.st_mode != expected:
            raise KMIPSecretStoreError(
                u._('Bad key file permissions found, expected 400 '
                    'for path: {file_path}').format(file_path=path)
            )

    def _normalize_secret(self, secret, secret_type):
        """Normalizes secret for use by KMIP plugin"""
        data = base64.b64decode(secret)
        if (self.pkcs1_only and
                secret_type in [ss.SecretType.PUBLIC, ss.SecretType.PRIVATE]):
            if secret_type == ss.SecretType.PUBLIC:
                data = get_public_key_der_pkcs1(data)
            elif secret_type == ss.SecretType.PRIVATE:
                data = get_private_key_der_pkcs1(data)
        elif secret_type in [ss.SecretType.PUBLIC,
                             ss.SecretType.PRIVATE,
                             ss.SecretType.CERTIFICATE]:
            data = translations.convert_pem_to_der(data, secret_type)
        return data

    def _denormalize_secret(self, secret, secret_type):
        """Converts secret back to the format expected by Barbican core"""
        data = secret
        if secret_type in [ss.SecretType.PUBLIC,
                           ss.SecretType.PRIVATE,
                           ss.SecretType.CERTIFICATE]:
            data = translations.convert_der_to_pem(data, secret_type)
        return base64.b64encode(data)



# Copyright (c) 2015 Cisco Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import unittest

from barbican.common import exception
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils


class WhenTestingProjectQuotasRepo(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingProjectQuotasRepo, self).setUp()
        self.project_quotas_repo = repositories.ProjectQuotasRepo()

        self.session = self.project_quotas_repo.get_session()

        self.project_1 = models.Project()
        self.project_1.id = '11111'
        self.project_1.external_id = '44444'
        self.project_1.save(session=self.session)
        self.project_2 = models.Project()
        self.project_2.id = '22222'
        self.project_2.external_id = '55555'
        self.project_2.save(session=self.session)
        self.project_3 = models.Project()
        self.project_3.id = '33333'
        self.project_3.external_id = '66666'
        self.project_3.save(session=self.session)

        self.parsed_project_quotas_1 = {
            'secrets': 101,
            'orders': 102,
            'containers': 103,
            'consumers': 105,
            'cas': 106}
        self.parsed_project_quotas_2 = {
            'secrets': 201,
            'orders': 202,
            'containers': 203,
            'consumers': 205,
            'cas': 206}
        self.parsed_project_quotas_3 = {
            'secrets': 301,
            'containers': 303,
            'consumers': 305}

    def test_get_list_of_one_project_quotas(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_1,
            session=self.session)
        self.session.commit()
        retrieved_project_quotas, offset, limit, total =\
            self.project_quotas_repo.get_by_create_date(session=self.session)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)
        self.assertEqual([self.project_1.id],
                         [s.project_id for s in retrieved_project_quotas])
        self.assertEqual([self.project_1.external_id],
                         [s.project.external_id for s
                          in retrieved_project_quotas])
        self.assertEqual([101],
                         [s.secrets for s in retrieved_project_quotas])
        self.assertEqual([102],
                         [s.orders for s in retrieved_project_quotas])
        self.assertEqual([103],
                         [s.containers for s in retrieved_project_quotas])
        self.assertEqual([105],
                         [s.consumers for s in retrieved_project_quotas])
        self.assertEqual([106],
                         [s.cas for s in retrieved_project_quotas])

    def test_get_list_of_two_project_quotas(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_1,
            session=self.session)
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_2.id,
            self.parsed_project_quotas_2,
            session=self.session)
        self.session.commit()
        retrieved_project_quotas, offset, limit, total =\
            self.project_quotas_repo.get_by_create_date(session=self.session)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(2, total)
        self.assertCountEqual([self.project_1.id, self.project_2.id],
                              [s.project_id for s in retrieved_project_quotas])
        self.assertCountEqual([self.project_1.external_id,
                               self.project_2.external_id],
                              [s.project.external_id for s
                               in retrieved_project_quotas])
        self.assertCountEqual([101, 201],
                              [s.secrets for s in retrieved_project_quotas])
        self.assertCountEqual([102, 202],
                              [s.orders for s in retrieved_project_quotas])
        self.assertCountEqual([103, 203],
                              [s.containers for s in retrieved_project_quotas])
        self.assertCountEqual([105, 205],
                              [s.consumers for s in retrieved_project_quotas])
        self.assertCountEqual([106, 206],
                              [s.cas for s in retrieved_project_quotas])

    def test_should_raise_get_list_of_zero_project_quotas(self):
        self.assertRaises(
            exception.NotFound,
            self.project_quotas_repo.get_by_create_date,
            session=self.session,
            suppress_exception=False)

    def test_should_suppress_get_list_of_zero_project_quotas(self):
        retrieved_project_quotas, offset, limit, total =\
            self.project_quotas_repo.get_by_create_date(
                session=self.session, suppress_exception=True)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(0, total)

    def test_get_specific_project_quotas(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_1,
            session=self.session)
        self.session.commit()
        retrieved_project_quotas =\
            self.project_quotas_repo.get_by_external_project_id(
                self.project_1.external_id, session=self.session)
        self.assertEqual(self.project_1.id,
                         retrieved_project_quotas.project_id)
        self.assertEqual(self.project_1.external_id,
                         retrieved_project_quotas.project.external_id)
        self.assertEqual(101, retrieved_project_quotas.secrets)
        self.assertEqual(102, retrieved_project_quotas.orders)
        self.assertEqual(103, retrieved_project_quotas.containers)
        self.assertEqual(105, retrieved_project_quotas.consumers)
        self.assertEqual(106, retrieved_project_quotas.cas)

    def test_project_quotas_with_some_defaults(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_3.id,
            self.parsed_project_quotas_3,
            session=self.session)
        self.session.commit()
        retrieved_project_quotas =\
            self.project_quotas_repo.get_by_external_project_id(
                self.project_3.external_id, session=self.session)
        self.assertEqual(self.project_3.id,
                         retrieved_project_quotas.project_id)
        self.assertEqual(self.project_3.external_id,
                         retrieved_project_quotas.project.external_id)
        self.assertEqual(301, retrieved_project_quotas.secrets)
        self.assertIsNone(retrieved_project_quotas.orders)
        self.assertEqual(303, retrieved_project_quotas.containers)
        self.assertEqual(305, retrieved_project_quotas.consumers)
        self.assertIsNone(retrieved_project_quotas.cas)

    def test_update_specific_project_quotas(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_1,
            session=self.session)
        self.session.commit()
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_2,
            session=self.session)
        self.session.commit()
        retrieved_project_quotas =\
            self.project_quotas_repo.get_by_external_project_id(
                self.project_1.external_id, session=self.session)
        self.assertEqual(self.project_1.id,
                         retrieved_project_quotas.project_id)
        self.assertEqual(self.project_1.external_id,
                         retrieved_project_quotas.project.external_id)
        self.assertEqual(201, retrieved_project_quotas.secrets)
        self.assertEqual(202, retrieved_project_quotas.orders)
        self.assertEqual(203, retrieved_project_quotas.containers)
        self.assertEqual(205, retrieved_project_quotas.consumers)
        self.assertEqual(206, retrieved_project_quotas.cas)

    def test_should_raise_get_missing_specific_project_quotas(self):
        self.assertRaises(
            exception.NotFound,
            self.project_quotas_repo.get_by_external_project_id,
            'dummy',
            suppress_exception=False,
            session=self.session)

    def test_should_suppress_get_missing_specific_project_quotas(self):
        retrieved_project_quotas =\
            self.project_quotas_repo.get_by_external_project_id(
                'dummy', suppress_exception=True, session=self.session)
        self.assertIsNone(retrieved_project_quotas)

    def test_get_by_create_date_nothing(self):
        retrieved_project_quotas, offset, limit, total =\
            self.project_quotas_repo.get_by_create_date(
                session=self.session, suppress_exception=True)
        self.assertEqual([], retrieved_project_quotas)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(0, total)

    def test_should_delete(self):
        self.project_quotas_repo.create_or_update_by_project_id(
            self.project_1.id,
            self.parsed_project_quotas_1,
            session=self.session)
        self.session.commit()
        self.project_quotas_repo.delete_by_external_project_id(
            self.project_1.external_id, session=self.session)

    def test_should_raise_delete_not_found(self):
        self.assertRaises(
            exception.NotFound,
            self.project_quotas_repo.delete_by_external_project_id,
            'dummy',
            session=self.session)

    def test_should_suppress_delete_not_found(self):
        self.project_quotas_repo.delete_by_external_project_id(
            'dummy', suppress_exception=True, session=self.session)

    def test_do_entity_name(self):
        self.assertEqual("ProjectQuotas",
                         self.project_quotas_repo._do_entity_name())

    def test_should_raise_not_found_get_by_entity_id(self):
        self.assertRaises(
            exception.NotFound,
            self.project_quotas_repo.get,
            'dummy',
            session=self.session)


if __name__ == '__main__':
    unittest.main()



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_log import versionutils
from oslo_policy import policy

from barbican.common.policies import base


deprecated_containers_post = policy.DeprecatedRule(
    name='containers:post',
    check_str='rule:admin_or_creator',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_containers_get = policy.DeprecatedRule(
    name='containers:get',
    check_str='rule:all_but_audit',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_container_get = policy.DeprecatedRule(
    name='container:get',
    check_str='rule:container_non_private_read or ' +
              'rule:container_project_creator or ' +
              'rule:container_project_admin or ' +
              'rule:container_acl_read',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_container_delete = policy.DeprecatedRule(
    name='container:delete',
    check_str='rule:container_project_admin or ' +
              'rule:container_project_creator',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_container_secret_post = policy.DeprecatedRule(
    name='container_secret:post',
    check_str='rule:container_project_admin or ' +
              'rule:container_project_creator or ' +
              'rule:container_project_creator_role and ' +
              'rule:container_non_private_read',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_container_secret_delete = policy.DeprecatedRule(
    name='container_secret:delete',
    check_str='rule:container_project_admin or ' +
              'rule:container_project_creator or ' +
              'rule:container_project_creator_role and ' +
              'rule:container_non_private_read',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    policy.DocumentedRuleDefault(
        name='containers:post',
        check_str='True:%(enforce_new_defaults)s and role:member',
        scope_types=['project'],
        description='Creates a container.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_containers_post
    ),
    policy.DocumentedRuleDefault(
        name='containers:get',
        check_str='True:%(enforce_new_defaults)s and role:member',
        scope_types=['project'],
        description='Lists a projects containers.',
        operations=[
            {
                'path': '/v1/containers',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_containers_get
    ),
    policy.DocumentedRuleDefault(
        name='container:get',
        check_str=(
            'True:%(enforce_new_defaults)s and '
            '(rule:container_project_admin or '
            '(rule:container_project_member and rule:container_owner) or '
            '(rule:container_project_member and '
            ' rule:container_is_not_private) or '
            'rule:container_acl_read)'),
        scope_types=['project'],
        description='Retrieves a single container.',
        operations=[
            {
                'path': '/v1/containers/{container-id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_container_get
    ),
    policy.DocumentedRuleDefault(
        name='container:delete',
        check_str=(
            'True:%(enforce_new_defaults)s and '
            '(rule:container_project_admin or '
            '(rule:container_project_member and rule:container_owner) or '
            '(rule:container_project_member and '
            ' rule:container_is_not_private))'),
        scope_types=['project'],
        description='Deletes a container.',
        operations=[
            {
                'path': '/v1/containers/{uuid}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_container_delete
    ),
    policy.DocumentedRuleDefault(
        name='container_secret:post',
        check_str=(
            'True:%(enforce_new_defaults)s and '
            '(rule:container_project_admin or '
            '(rule:container_project_member and rule:container_owner) or '
            '(rule:container_project_member and '
            ' rule:container_is_not_private))'),
        scope_types=['project'],
        description='Add a secret to an existing container.',
        operations=[
            {
                'path': '/v1/containers/{container-id}/secrets',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_container_secret_post
    ),
    policy.DocumentedRuleDefault(
        name='container_secret:delete',
        check_str=(
            'True:%(enforce_new_defaults)s and '
            '(rule:container_project_admin or '
            '(rule:container_project_member and rule:container_owner) or '
            '(rule:container_project_member and '
            ' rule:container_is_not_private))'),
        scope_types=['project'],
        description='Remove a secret from a container.',
        operations=[
            {
                'path': '/v1/containers/{container-id}/secrets/{secret-id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_container_secret_delete
    ),
]


def list_rules():
    return rules



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from barbican.api.middleware import simple


class WhenTestingSimpleMiddleware(unittest.TestCase):

    def setUp(self):
        self.app = mock.MagicMock()
        self.middle = simple.SimpleFilter(self.app)
        self.req = mock.MagicMock()

    def test_should_process_request(self):
        self.middle.process_request(self.req)



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.model import models
from barbican.model import repositories as repos
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class OrderBarbicanMetadatum(base.BarbicanObject,
                             base.BarbicanPersistentObject,
                             object_base.VersionedObjectDictCompat):
    """This class represents OrderBarbicanMetadatum object"""

    fields = {
        'order_id': fields.StringField(),
        'key': fields.StringField(),
        'value': fields.StringField()
    }

    db_model = models.OrderBarbicanMetadatum
    db_repo = repos.get_order_barbican_meta_repository()



# Copyright 2015 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

# Initial operations for agent management extension
# This module only manages the 'agents' table. Binding tables are created
# in the modules for relevant resources


from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'kek_data',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('deleted_at', sa.DateTime(), nullable=True),
        sa.Column('deleted', sa.Boolean(), nullable=False),
        sa.Column('status', sa.String(length=20), nullable=False),
        sa.Column('plugin_name', sa.String(length=255), nullable=False),
        sa.Column('kek_label', sa.String(length=255), nullable=True),
        sa.Column('tenant_id', sa.String(length=36), nullable=False),
        sa.Column('active', sa.Boolean(), nullable=False),
        sa.Column('bind_completed', sa.Boolean(), nullable=False),
        sa.Column('algorithm', sa.String(length=255), nullable=True),
        sa.Column('bit_length', sa.Integer(), nullable=True),
        sa.Column('mode', sa.String(length=255), nullable=True),
        sa.Column('plugin_meta', sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(['tenant_id'], ['tenants.id'],),
        sa.PrimaryKeyConstraint('id')
    )



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbican import i18n as u
from barbican.tasks import common
from barbican.tests import utils


class WhenUsingFollowOnProcessingStatusDTO(utils.BaseTestCase):
    """Test using the :class:`WhenUsingFollowOnProcessingStatusDTO` class."""

    def setUp(self):
        super(WhenUsingFollowOnProcessingStatusDTO, self).setUp()

        self.target = common.FollowOnProcessingStatusDTO()

    def test_should_have_expected_defaults(self):
        self.assertEqual(
            common.RetryTasks.NO_ACTION_REQUIRED, self.target.retry_task)
        self.assertEqual(u._('Unknown'), self.target.status)
        self.assertEqual(u._('Unknown'), self.target.status_message)
        self.assertEqual(common.RETRY_MSEC_DEFAULT, self.target.retry_msec)
        self.assertFalse(self.target.is_follow_on_needed())

    def test_should_indicate_no_follow_on_with_no_retry_task(self):
        self.target.retry_task = None

        self.assertFalse(self.target.is_follow_on_needed())

    def test_should_indicate_follow_on_when_retry_task_provided(self):
        self.target.retry_task = common.RetryTasks.INVOKE_SAME_TASK

        self.assertTrue(self.target.is_follow_on_needed())



# Copyright (c) 2013-2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Task resources for the Barbican API.
"""
import abc

from barbican import api
from barbican.common import utils
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as rep
from barbican.plugin import resources as plugin
from barbican.tasks import common


LOG = utils.getLogger(__name__)


class BaseTask(object, metaclass=abc.ABCMeta):
    """Base asynchronous task."""

    @abc.abstractmethod
    def get_name(self):
        """Localized task name

        A hook method to return a short localized name for this task.
        The returned name in the form 'u.('Verb Noun')'. For example:
            u._('Create Secret')
        """

    def process_and_suppress_exceptions(self, *args, **kwargs):
        """Invokes the process() template method, suppressing all exceptions.

        TODO(john-wood-w) This method suppresses exceptions for flows that
        do not want to rollback database modifications in reaction to such
        exceptions, as this could also rollback the marking of the entity
        (eg. order) in the ERROR status via the handle_error() call below.
        For Liberty, we might want to consider a workflow manager instead of
        these process_xxxx() method as shown here:
        https://gist.github.com/jfwood/a8130265b0db3c793ec8

        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: Returns :class:`FollowOnProcessingStatusDTO` if follow-on
                 processing (such as retrying this or another task) is
                 required, otherwise a None return indicates that no
                 follow-on processing is required.
        """
        try:
            return self.process(*args, **kwargs)
        except Exception:
            LOG.exception("Suppressing exception while trying to "
                          "process task '%s'.", self.get_name())

    def process(self, *args, **kwargs):
        """A template method for all asynchronous tasks.

        This method should not be overridden by sub-classes. Rather the
        abstract methods below should be overridden.

        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: Returns :class:`FollowOnProcessingStatusDTO` if follow-on
                 processing (such as retrying this or another task) is
                 required, otherwise a None return indicates that no
                 follow-on processing is required.
        """
        name = self.get_name()
        result = None

        # Retrieve the target entity (such as an models.Order instance).
        try:
            entity = self.retrieve_entity(*args, **kwargs)
        except Exception:
            # Serious error!
            LOG.exception("Could not retrieve information needed to "
                          "process task '%s'.", name)
            raise

        # Process the target entity.
        try:
            result = self.handle_processing(entity, *args, **kwargs)
        except Exception as e_orig:
            LOG.exception("Could not perform processing for task '%s'.", name)

            # Handle failure to process entity.
            try:
                status, message = api.generate_safe_exception_message(name,
                                                                      e_orig)
                self.handle_error(entity, status, message, e_orig,
                                  *args, **kwargs)
            except Exception:
                LOG.exception("Problem handling an error for task '%s', "
                              "raising original exception.", name)
            raise e_orig

        # Handle successful conclusion of processing.
        try:
            self.handle_success(entity, result, *args, **kwargs)
        except Exception:
            LOG.exception("Could not process after successfully "
                          "executing task '%s'.", name)
            raise
        return result

    @abc.abstractmethod
    def retrieve_entity(self, *args, **kwargs):
        """A hook method to retrieve an entity for processing.

        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: Entity instance to process in subsequent hook methods.
        """

    @abc.abstractmethod
    def handle_processing(self, entity, *args, **kwargs):
        """A hook method to handle processing on behalf of an entity.

        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: None if no follow on processing is needed for this task,
                 otherwise a :class:`FollowOnProcessingStatusDTO` instance
                 with information on how to process this task into the future.
        """

    @abc.abstractmethod
    def handle_error(self, entity, status, message, exception,
                     *args, **kwargs):
        """A hook method to deal with errors seen during processing.

        This method could be used to mark entity as being in error, and/or
        to record an error cause.

        :param entity: Entity retrieved from _retrieve_entity() above.
        :param status: Status code for exception.
        :param message: Reason/message for the exception.
        :param exception: Exception raised from handle_processing() above.
        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: None
        """

    @abc.abstractmethod
    def handle_success(self, entity, result, *args, **kwargs):
        """A hook method to post-process after successful entity processing.

        This method could be used to mark entity as being active, or to
        add information/references to the entity.

        :param entity: Entity retrieved from _retrieve_entity() above.
        :param result: A :class:`FollowOnProcessingStatusDTO` instance
                       representing processing result status, None implies
                       that no follow on processing is required.
        :param args: List of arguments passed in from the client.
        :param kwargs: Dict of arguments passed in from the client.
        :return: None
        """


class _OrderTaskHelper(object):
    """Supports order-related BaseTask operations.

    BaseTask sub-classes can delegate to an instance of this class to perform
    common order-related operations.
    """
    def __init__(self):
        self.order_repo = rep.get_order_repository()

    def retrieve_entity(self, order_id, external_project_id, *args, **kwargs):
        """Retrieve an order entity by its PK ID."""
        return self.order_repo.get(
            entity_id=order_id,
            external_project_id=external_project_id)

    def handle_error(self, order, status, message, exception,
                     *args, **kwargs):
        """Stamp the order entity as terminated due to an error."""
        order.status = models.States.ERROR
        order.error_status_code = status
        order.set_error_reason_safely(message)
        self.order_repo.save(order)

    def handle_success(self, order, result, *args, **kwargs):
        """Handle if the order entity is terminated or else long running.

        The 'result' argument (if present) indicates if a order should now be
        terminated due to it being completed, or else should be held in the
        PENDING state due to follow on workflow processing. If 'result' is not
        provided, the order is presumed completed.
        """
        is_follow_on_needed = False
        sub_status = None
        sub_status_message = None
        if result:
            is_follow_on_needed = result.is_follow_on_needed()
            sub_status = result.status
            sub_status_message = result.status_message

        if not is_follow_on_needed:
            order.status = models.States.ACTIVE
        else:
            order.status = models.States.PENDING

        if sub_status:
            order.set_sub_status_safely(sub_status)

        if sub_status_message:
            order.set_sub_status_message_safely(sub_status_message)

        self.order_repo.save(order)


class BeginTypeOrder(BaseTask):
    """Handles beginning processing of a TypeOrder."""

    def get_name(self):
        return u._('Process TypeOrder')

    def __init__(self):
        super(BeginTypeOrder, self).__init__()
        LOG.debug('Creating BeginTypeOrder task processor')
        self.project_repo = rep.get_project_repository()
        self.helper = _OrderTaskHelper()

    def retrieve_entity(self, *args, **kwargs):
        return self.helper.retrieve_entity(*args, **kwargs)

    def handle_processing(self, order, *args, **kwargs):
        return self.handle_order(order)

    def handle_order(self, order):
        """Handle secret creation using meta info.

        If type is key
            create secret
        if type is asymmetric
            create secrets
            create containers
        if type is certificate
            TBD
        :param order: Order to process.
        :return: None if no follow on processing is needed for this task,
                 otherwise a :class:`FollowOnProcessingStatusDTO` instance
                 with information on how to process this task into the future.
        """
        result_follow_on = common.FollowOnProcessingStatusDTO()

        order_info = order.to_dict_fields()
        order_type = order_info.get('type')
        meta_info = order_info.get('meta')
        if order_info.get('creator_id'):
            meta_info.setdefault('creator_id', order_info.get('creator_id'))

        # Retrieve the project.
        project = self.project_repo.get(order.project_id)

        if order_type == models.OrderType.KEY:
            # Create Secret
            new_secret = plugin.generate_secret(
                meta_info,
                meta_info.get('payload_content_type',
                              'application/octet-stream'),
                project
            )
            order.secret_id = new_secret.id
            LOG.debug("...done creating keys order's secret.")
        elif order_type == models.OrderType.ASYMMETRIC:
            # Create asymmetric Secret
            new_container = plugin.generate_asymmetric_secret(
                meta_info,
                meta_info.get('payload_content_type',
                              'application/octet-stream'),
                project)
            order.container_id = new_container.id
            LOG.debug("...done creating asymmetric order's secret.")
        else:
            raise NotImplementedError(
                u._('Order type "{order_type}" not implemented.').format(
                    order_type=order_type))

        return result_follow_on

    def handle_error(self, order, status, message, exception,
                     *args, **kwargs):
        self.helper.handle_error(
            order, status, message, exception, *args, **kwargs)

    def handle_success(self, order, result, *args, **kwargs):
        self.helper.handle_success(
            order, result, *args, **kwargs)
